//line php7/php7.y:2
package php7

import __yyfmt__ "fmt"

//line php7/php7.y:2
import (
	"strconv"
	"strings"

	"github.com/z7zmey/php-parser/comment"
	"github.com/z7zmey/php-parser/node"
	"github.com/z7zmey/php-parser/node/expr"
	"github.com/z7zmey/php-parser/node/expr/assign"
	"github.com/z7zmey/php-parser/node/expr/binary"
	"github.com/z7zmey/php-parser/node/expr/cast"
	"github.com/z7zmey/php-parser/node/name"
	"github.com/z7zmey/php-parser/node/scalar"
	"github.com/z7zmey/php-parser/node/stmt"
	"github.com/z7zmey/php-parser/scanner"
)

//line php7/php7.y:22
type yySymType struct {
	yys   int
	node  node.Node
	token *scanner.Token
	list  []node.Node
	str   string

	ClassExtends     *stmt.ClassExtends
	ClassImplements  *stmt.ClassImplements
	InterfaceExtends *stmt.InterfaceExtends
	ClosureUse       *expr.ClosureUse
}

const T_INCLUDE = 57346
const T_INCLUDE_ONCE = 57347
const T_EXIT = 57348
const T_IF = 57349
const T_LNUMBER = 57350
const T_DNUMBER = 57351
const T_STRING = 57352
const T_STRING_VARNAME = 57353
const T_VARIABLE = 57354
const T_NUM_STRING = 57355
const T_INLINE_HTML = 57356
const T_CHARACTER = 57357
const T_BAD_CHARACTER = 57358
const T_ENCAPSED_AND_WHITESPACE = 57359
const T_CONSTANT_ENCAPSED_STRING = 57360
const T_ECHO = 57361
const T_DO = 57362
const T_WHILE = 57363
const T_ENDWHILE = 57364
const T_FOR = 57365
const T_ENDFOR = 57366
const T_FOREACH = 57367
const T_ENDFOREACH = 57368
const T_DECLARE = 57369
const T_ENDDECLARE = 57370
const T_AS = 57371
const T_SWITCH = 57372
const T_ENDSWITCH = 57373
const T_CASE = 57374
const T_DEFAULT = 57375
const T_BREAK = 57376
const T_CONTINUE = 57377
const T_GOTO = 57378
const T_FUNCTION = 57379
const T_CONST = 57380
const T_RETURN = 57381
const T_TRY = 57382
const T_CATCH = 57383
const T_FINALLY = 57384
const T_THROW = 57385
const T_USE = 57386
const T_INSTEADOF = 57387
const T_GLOBAL = 57388
const T_VAR = 57389
const T_UNSET = 57390
const T_ISSET = 57391
const T_EMPTY = 57392
const T_HALT_COMPILER = 57393
const T_CLASS = 57394
const T_TRAIT = 57395
const T_INTERFACE = 57396
const T_EXTENDS = 57397
const T_IMPLEMENTS = 57398
const T_OBJECT_OPERATOR = 57399
const T_DOUBLE_ARROW = 57400
const T_LIST = 57401
const T_ARRAY = 57402
const T_CALLABLE = 57403
const T_CLASS_C = 57404
const T_TRAIT_C = 57405
const T_METHOD_C = 57406
const T_FUNC_C = 57407
const T_LINE = 57408
const T_FILE = 57409
const T_COMMENT = 57410
const T_DOC_COMMENT = 57411
const T_OPEN_TAG = 57412
const T_OPEN_TAG_WITH_ECHO = 57413
const T_CLOSE_TAG = 57414
const T_WHITESPACE = 57415
const T_START_HEREDOC = 57416
const T_END_HEREDOC = 57417
const T_DOLLAR_OPEN_CURLY_BRACES = 57418
const T_CURLY_OPEN = 57419
const T_PAAMAYIM_NEKUDOTAYIM = 57420
const T_NAMESPACE = 57421
const T_NS_C = 57422
const T_DIR = 57423
const T_NS_SEPARATOR = 57424
const T_ELLIPSIS = 57425
const T_EVAL = 57426
const T_REQUIRE = 57427
const T_REQUIRE_ONCE = 57428
const T_LOGICAL_OR = 57429
const T_LOGICAL_XOR = 57430
const T_LOGICAL_AND = 57431
const T_INSTANCEOF = 57432
const T_NEW = 57433
const T_CLONE = 57434
const T_ELSEIF = 57435
const T_ELSE = 57436
const T_ENDIF = 57437
const T_PRINT = 57438
const T_YIELD = 57439
const T_STATIC = 57440
const T_ABSTRACT = 57441
const T_FINAL = 57442
const T_PRIVATE = 57443
const T_PROTECTED = 57444
const T_PUBLIC = 57445
const T_INC = 57446
const T_DEC = 57447
const T_YIELD_FROM = 57448
const T_INT_CAST = 57449
const T_DOUBLE_CAST = 57450
const T_STRING_CAST = 57451
const T_ARRAY_CAST = 57452
const T_OBJECT_CAST = 57453
const T_BOOL_CAST = 57454
const T_UNSET_CAST = 57455
const T_COALESCE = 57456
const T_SPACESHIP = 57457
const T_NOELSE = 57458
const T_PLUS_EQUAL = 57459
const T_MINUS_EQUAL = 57460
const T_MUL_EQUAL = 57461
const T_POW_EQUAL = 57462
const T_DIV_EQUAL = 57463
const T_CONCAT_EQUAL = 57464
const T_MOD_EQUAL = 57465
const T_AND_EQUAL = 57466
const T_OR_EQUAL = 57467
const T_XOR_EQUAL = 57468
const T_SL_EQUAL = 57469
const T_SR_EQUAL = 57470
const T_BOOLEAN_OR = 57471
const T_BOOLEAN_AND = 57472
const T_POW = 57473
const T_SL = 57474
const T_SR = 57475
const T_IS_IDENTICAL = 57476
const T_IS_NOT_IDENTICAL = 57477
const T_IS_EQUAL = 57478
const T_IS_NOT_EQUAL = 57479
const T_IS_SMALLER_OR_EQUAL = 57480
const T_IS_GREATER_OR_EQUAL = 57481

var yyToknames = [...]string{
	"$end",
	"error",
	"$unk",
	"T_INCLUDE",
	"T_INCLUDE_ONCE",
	"T_EXIT",
	"T_IF",
	"T_LNUMBER",
	"T_DNUMBER",
	"T_STRING",
	"T_STRING_VARNAME",
	"T_VARIABLE",
	"T_NUM_STRING",
	"T_INLINE_HTML",
	"T_CHARACTER",
	"T_BAD_CHARACTER",
	"T_ENCAPSED_AND_WHITESPACE",
	"T_CONSTANT_ENCAPSED_STRING",
	"T_ECHO",
	"T_DO",
	"T_WHILE",
	"T_ENDWHILE",
	"T_FOR",
	"T_ENDFOR",
	"T_FOREACH",
	"T_ENDFOREACH",
	"T_DECLARE",
	"T_ENDDECLARE",
	"T_AS",
	"T_SWITCH",
	"T_ENDSWITCH",
	"T_CASE",
	"T_DEFAULT",
	"T_BREAK",
	"T_CONTINUE",
	"T_GOTO",
	"T_FUNCTION",
	"T_CONST",
	"T_RETURN",
	"T_TRY",
	"T_CATCH",
	"T_FINALLY",
	"T_THROW",
	"T_USE",
	"T_INSTEADOF",
	"T_GLOBAL",
	"T_VAR",
	"T_UNSET",
	"T_ISSET",
	"T_EMPTY",
	"T_HALT_COMPILER",
	"T_CLASS",
	"T_TRAIT",
	"T_INTERFACE",
	"T_EXTENDS",
	"T_IMPLEMENTS",
	"T_OBJECT_OPERATOR",
	"T_DOUBLE_ARROW",
	"T_LIST",
	"T_ARRAY",
	"T_CALLABLE",
	"T_CLASS_C",
	"T_TRAIT_C",
	"T_METHOD_C",
	"T_FUNC_C",
	"T_LINE",
	"T_FILE",
	"T_COMMENT",
	"T_DOC_COMMENT",
	"T_OPEN_TAG",
	"T_OPEN_TAG_WITH_ECHO",
	"T_CLOSE_TAG",
	"T_WHITESPACE",
	"T_START_HEREDOC",
	"T_END_HEREDOC",
	"T_DOLLAR_OPEN_CURLY_BRACES",
	"T_CURLY_OPEN",
	"T_PAAMAYIM_NEKUDOTAYIM",
	"T_NAMESPACE",
	"T_NS_C",
	"T_DIR",
	"T_NS_SEPARATOR",
	"T_ELLIPSIS",
	"T_EVAL",
	"T_REQUIRE",
	"T_REQUIRE_ONCE",
	"T_LOGICAL_OR",
	"T_LOGICAL_XOR",
	"T_LOGICAL_AND",
	"T_INSTANCEOF",
	"T_NEW",
	"T_CLONE",
	"T_ELSEIF",
	"T_ELSE",
	"T_ENDIF",
	"T_PRINT",
	"T_YIELD",
	"T_STATIC",
	"T_ABSTRACT",
	"T_FINAL",
	"T_PRIVATE",
	"T_PROTECTED",
	"T_PUBLIC",
	"T_INC",
	"T_DEC",
	"T_YIELD_FROM",
	"T_INT_CAST",
	"T_DOUBLE_CAST",
	"T_STRING_CAST",
	"T_ARRAY_CAST",
	"T_OBJECT_CAST",
	"T_BOOL_CAST",
	"T_UNSET_CAST",
	"T_COALESCE",
	"T_SPACESHIP",
	"T_NOELSE",
	"T_PLUS_EQUAL",
	"T_MINUS_EQUAL",
	"T_MUL_EQUAL",
	"T_POW_EQUAL",
	"T_DIV_EQUAL",
	"T_CONCAT_EQUAL",
	"T_MOD_EQUAL",
	"T_AND_EQUAL",
	"T_OR_EQUAL",
	"T_XOR_EQUAL",
	"T_SL_EQUAL",
	"T_SR_EQUAL",
	"T_BOOLEAN_OR",
	"T_BOOLEAN_AND",
	"T_POW",
	"T_SL",
	"T_SR",
	"T_IS_IDENTICAL",
	"T_IS_NOT_IDENTICAL",
	"T_IS_EQUAL",
	"T_IS_NOT_EQUAL",
	"T_IS_SMALLER_OR_EQUAL",
	"T_IS_GREATER_OR_EQUAL",
	"'\"'",
	"'`'",
	"'{'",
	"'}'",
	"';'",
	"':'",
	"'('",
	"')'",
	"'['",
	"']'",
	"'?'",
	"'&'",
	"'-'",
	"'+'",
	"'!'",
	"'~'",
	"'@'",
	"'$'",
	"','",
	"'|'",
	"'='",
	"'^'",
	"'*'",
	"'/'",
	"'%'",
	"'<'",
	"'>'",
	"'.'",
}
var yyStatenames = [...]string{}

const yyEofCode = 1
const yyErrCode = 2
const yyInitialStackSize = 16

//line php7/php7.y:5346

//line yacctab:1
var yyExca = [...]int{
	-1, 1,
	1, -1,
	-2, 0,
	-1, 2,
	1, 1,
	-2, 0,
	-1, 44,
	57, 422,
	78, 422,
	142, 422,
	148, 422,
	-2, 417,
	-1, 48,
	146, 425,
	-2, 434,
	-1, 84,
	57, 424,
	78, 424,
	142, 424,
	146, 427,
	148, 424,
	-2, 412,
	-1, 107,
	78, 385,
	-2, 414,
	-1, 229,
	57, 422,
	78, 422,
	142, 422,
	148, 422,
	-2, 313,
	-1, 232,
	146, 427,
	-2, 424,
	-1, 235,
	57, 422,
	78, 422,
	142, 422,
	148, 422,
	-2, 315,
	-1, 354,
	115, 0,
	134, 0,
	135, 0,
	136, 0,
	137, 0,
	-2, 337,
	-1, 355,
	115, 0,
	134, 0,
	135, 0,
	136, 0,
	137, 0,
	-2, 338,
	-1, 356,
	115, 0,
	134, 0,
	135, 0,
	136, 0,
	137, 0,
	-2, 339,
	-1, 357,
	115, 0,
	134, 0,
	135, 0,
	136, 0,
	137, 0,
	-2, 340,
	-1, 358,
	138, 0,
	139, 0,
	165, 0,
	166, 0,
	-2, 341,
	-1, 359,
	138, 0,
	139, 0,
	165, 0,
	166, 0,
	-2, 342,
	-1, 360,
	138, 0,
	139, 0,
	165, 0,
	166, 0,
	-2, 343,
	-1, 361,
	138, 0,
	139, 0,
	165, 0,
	166, 0,
	-2, 344,
	-1, 362,
	115, 0,
	134, 0,
	135, 0,
	136, 0,
	137, 0,
	-2, 345,
	-1, 369,
	147, 163,
	158, 163,
	-2, 422,
	-1, 413,
	147, 462,
	149, 462,
	158, 462,
	-2, 422,
	-1, 417,
	57, 423,
	78, 423,
	142, 423,
	146, 426,
	148, 423,
	-2, 347,
	-1, 431,
	146, 448,
	-2, 415,
	-1, 432,
	146, 450,
	-2, 440,
	-1, 511,
	146, 448,
	-2, 416,
	-1, 512,
	146, 450,
	-2, 441,
	-1, 571,
	147, 213,
	-2, 218,
	-1, 596,
	146, 426,
	-2, 423,
	-1, 648,
	147, 213,
	-2, 218,
	-1, 653,
	147, 183,
	-2, 422,
	-1, 661,
	147, 213,
	-2, 218,
	-1, 686,
	147, 461,
	149, 461,
	158, 461,
	-2, 422,
	-1, 719,
	147, 184,
	-2, 422,
	-1, 739,
	12, 265,
	-2, 268,
	-1, 753,
	93, 208,
	94, 208,
	95, 208,
	-2, 0,
	-1, 778,
	147, 183,
	-2, 422,
	-1, 780,
	147, 186,
	-2, 396,
	-1, 800,
	93, 209,
	94, 209,
	95, 209,
	-2, 0,
	-1, 849,
	31, 199,
	32, 199,
	33, 199,
	143, 199,
	-2, 0,
	-1, 882,
	29, 76,
	-2, 80,
	-1, 885,
	31, 198,
	32, 198,
	33, 198,
	143, 198,
	-2, 0,
	-1, 914,
	147, 213,
	-2, 218,
}

const yyPrivate = 57344

const yyLast = 7805

var yyAct = [...]int{

	28, 130, 825, 838, 662, 861, 376, 578, 741, 436,
	874, 107, 837, 793, 774, 666, 811, 675, 785, 126,
	652, 633, 138, 138, 138, 664, 221, 151, 711, 565,
	700, 313, 667, 322, 5, 186, 532, 632, 368, 573,
	405, 514, 522, 80, 223, 317, 150, 378, 9, 316,
	132, 189, 8, 225, 228, 128, 143, 236, 237, 238,
	239, 240, 147, 2, 241, 242, 243, 244, 245, 246,
	247, 127, 250, 125, 513, 258, 259, 260, 315, 314,
	137, 7, 6, 430, 264, 870, 864, 684, 835, 273,
	274, 854, 276, 277, 842, 590, 841, 269, 230, 230,
	82, 334, 309, 140, 141, 406, 892, 105, 868, 832,
	677, 541, 335, 568, 331, 329, 307, 893, 677, 254,
	759, 111, 869, 833, 308, 302, 336, 704, 332, 330,
	308, 301, 319, 115, 121, 728, 324, 325, 84, 626,
	621, 44, 306, 105, 566, 302, 556, 411, 105, 285,
	287, 190, 862, 730, 337, 338, 339, 340, 341, 342,
	343, 344, 345, 346, 347, 348, 349, 350, 351, 352,
	353, 354, 355, 356, 357, 358, 359, 360, 361, 362,
	295, 364, 366, 219, 370, 321, 266, 372, 780, 693,
	689, 269, 610, 232, 232, 606, 229, 235, 333, 79,
	607, 380, 388, 390, 391, 392, 393, 394, 395, 396,
	397, 398, 399, 400, 401, 597, 585, 402, 138, 404,
	424, 225, 179, 410, 144, 600, 230, 603, 601, 384,
	261, 926, 415, 888, 290, 225, 292, 407, 855, 820,
	179, 299, 826, 819, 305, 809, 801, 409, 518, 784,
	138, 773, 106, 727, 717, 698, 371, 425, 696, 688,
	650, 416, 138, 165, 363, 638, 431, 511, 218, 519,
	230, 628, 523, 524, 217, 598, 525, 589, 283, 914,
	266, 165, 168, 169, 529, 270, 291, 533, 106, 225,
	117, 863, 105, 106, 164, 166, 167, 828, 403, 813,
	812, 230, 163, 162, 782, 720, 284, 687, 661, 289,
	551, 423, 164, 166, 167, 535, 648, 161, 286, 646,
	306, 232, 647, 751, 369, 571, 554, 5, 552, 414,
	418, 385, 268, 517, 383, 561, 288, 151, 275, 516,
	560, 9, 429, 520, 179, 8, 272, 510, 271, 249,
	220, 216, 184, 183, 117, 182, 105, 136, 538, 134,
	135, 387, 112, 131, 550, 232, 564, 602, 413, 294,
	113, 293, 420, 421, 7, 6, 544, 188, 233, 270,
	920, 903, 570, 562, 580, 165, 581, 558, 577, 582,
	583, 575, 768, 769, 579, 559, 232, 902, 420, 427,
	421, 421, 420, 930, 887, 929, 163, 162, 588, 850,
	821, 179, 225, 592, 815, 225, 164, 166, 167, 408,
	808, 161, 543, 134, 546, 572, 112, 768, 769, 609,
	765, 752, 716, 714, 612, 712, 710, 106, 707, 555,
	540, 595, 233, 537, 386, 374, 328, 587, 155, 157,
	156, 179, 165, 168, 169, 327, 591, 326, 296, 175,
	177, 117, 432, 512, 536, 844, 807, 804, 802, 761,
	536, 536, 611, 163, 162, 181, 178, 608, 909, 886,
	859, 117, 536, 164, 166, 167, 174, 176, 161, 857,
	153, 154, 165, 168, 169, 170, 171, 172, 173, 175,
	177, 106, 179, 814, 803, 813, 812, 790, 783, 733,
	255, 180, 159, 163, 162, 422, 676, 138, 616, 576,
	158, 185, 160, 164, 166, 167, 174, 176, 161, 419,
	144, 669, 670, 114, 165, 627, 253, 304, 767, 198,
	200, 199, 789, 165, 114, 120, 304, 191, 196, 197,
	134, 620, 613, 112, 117, 642, 324, 644, 637, 617,
	304, 625, 117, 117, 649, 306, 304, 634, 619, 289,
	5, 304, 906, 718, 256, 257, 574, 515, 263, 117,
	673, 262, 674, 671, 9, 640, 579, 379, 8, 643,
	289, 685, 904, 116, 77, 78, 702, 615, 382, 631,
	658, 786, 636, 630, 146, 651, 123, 124, 691, 905,
	230, 230, 672, 549, 547, 320, 681, 7, 6, 542,
	690, 668, 523, 134, 303, 894, 112, 533, 117, 46,
	105, 255, 724, 725, 545, 300, 110, 230, 528, 794,
	280, 281, 829, 708, 745, 746, 747, 744, 743, 742,
	701, 122, 148, 715, 117, 705, 634, 225, 148, 695,
	671, 722, 703, 699, 697, 298, 726, 546, 192, 546,
	796, 795, 117, 671, 706, 604, 713, 146, 109, 732,
	671, 123, 124, 721, 753, 754, 635, 117, 225, 749,
	816, 768, 769, 129, 748, 256, 257, 134, 230, 119,
	112, 731, 757, 548, 536, 232, 232, 255, 369, 653,
	750, 324, 422, 377, 755, 614, 233, 255, 408, 618,
	375, 225, 701, 195, 634, 194, 762, 771, 768, 769,
	758, 193, 232, 760, 187, 686, 763, 669, 670, 777,
	671, 770, 1, 772, 766, 38, 797, 779, 798, 775,
	579, 791, 546, 800, 787, 739, 134, 546, 546, 112,
	736, 737, 230, 663, 231, 531, 234, 222, 521, 810,
	527, 256, 257, 860, 738, 106, 367, 740, 818, 805,
	526, 256, 257, 873, 145, 824, 792, 142, 827, 323,
	806, 149, 817, 232, 823, 836, 719, 660, 822, 255,
	251, 834, 381, 639, 278, 255, 845, 928, 830, 645,
	422, 255, 840, 849, 224, 43, 282, 42, 16, 15,
	851, 843, 599, 267, 546, 49, 546, 848, 745, 746,
	747, 744, 743, 742, 853, 48, 867, 108, 671, 255,
	50, 846, 858, 871, 252, 83, 880, 866, 839, 884,
	885, 775, 81, 72, 879, 248, 62, 232, 265, 890,
	778, 891, 279, 256, 257, 895, 61, 889, 878, 256,
	257, 898, 877, 883, 896, 256, 257, 880, 899, 876,
	875, 546, 897, 735, 901, 879, 45, 665, 908, 723,
	657, 310, 118, 297, 3, 435, 788, 729, 0, 912,
	0, 0, 0, 256, 257, 913, 0, 0, 0, 0,
	919, 922, 915, 0, 918, 917, 579, 0, 0, 923,
	0, 0, 924, 0, 0, 0, 671, 0, 0, 927,
	0, 4, 931, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 13, 23, 33,
	0, 0, 34, 12, 0, 24, 0, 29, 86, 87,
	10, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 738, 0, 0,
	740, 0, 0, 103, 0, 0, 0, 0, 11, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 214, 215, 0, 0, 0, 0,
	0, 745, 746, 747, 744, 743, 742, 202, 203, 204,
	205, 206, 207, 208, 209, 210, 211, 212, 213, 104,
	73, 14, 629, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 4, 0, 88,
	89, 70, 47, 93, 94, 36, 865, 105, 0, 27,
	201, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 13, 23, 33, 0, 0, 34, 12,
	0, 24, 0, 29, 86, 87, 10, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 738, 0, 0, 740, 0, 0, 103,
	0, 0, 0, 0, 11, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 745, 746, 747,
	744, 743, 742, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 104, 73, 14, 539, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 4, 0, 88, 89, 70, 47, 93,
	94, 36, 831, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 13,
	23, 33, 0, 0, 34, 12, 0, 24, 0, 29,
	86, 87, 10, 39, 40, 41, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 738,
	0, 0, 740, 0, 0, 103, 0, 0, 0, 0,
	11, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	77, 78, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 0, 0,
	0, 0, 0, 745, 746, 747, 744, 743, 742, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 104, 73, 14, 0, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 312,
	0, 88, 89, 70, 47, 93, 94, 36, 799, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 318, 39,
	40, 41, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 738, 0, 0, 740, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 25, 77, 78, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 0, 0, 0, 0, 0, 745,
	746, 747, 744, 743, 742, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 104, 73, 14,
	932, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 312, 0, 88, 89, 70,
	47, 93, 94, 36, 734, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 318, 39, 40, 41, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 77, 78, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 104, 73, 14, 925, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 312, 0, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 86, 87,
	318, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 104,
	73, 14, 921, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 312, 0, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 318, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 104, 73, 14, 911, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 312, 0, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 318, 39, 40, 41, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	77, 78, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 104, 73, 14, 910, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 312,
	0, 88, 89, 70, 47, 93, 94, 36, 0, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 907, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 318, 39,
	40, 41, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 25, 77, 78, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 104, 73, 14,
	0, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 312, 0, 88, 89, 70,
	47, 93, 94, 36, 0, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 318, 39, 40, 41, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 77, 78, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 104, 73, 14, 856, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 312, 0, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 852, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 86, 87,
	318, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 104,
	73, 14, 0, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 312, 0, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 781, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 318, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 104, 73, 14, 0, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 312, 0, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 764, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 318, 39, 40, 41, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	77, 78, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 104, 73, 14, 0, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 312,
	0, 88, 89, 70, 47, 93, 94, 36, 0, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 318, 39,
	40, 41, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 680, 74, 75, 25, 77, 78, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 104, 73, 14,
	0, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 312, 0, 88, 89, 70,
	47, 93, 94, 36, 0, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 318, 39, 40, 41, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 77, 78, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 104, 73, 14, 569, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 312, 0, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 86, 87,
	318, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 104,
	73, 14, 311, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 312, 0, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 318, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 104, 73, 14, 0, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 443, 444, 454, 455, 0, 0, 434,
	0, 105, 0, 0, 0, 0, 0, 0, 459, 460,
	461, 462, 463, 464, 465, 466, 467, 468, 469, 489,
	490, 491, 492, 493, 481, 482, 483, 484, 485, 470,
	471, 472, 473, 474, 475, 476, 477, 478, 479, 480,
	0, 501, 499, 500, 496, 497, 0, 0, 488, 494,
	495, 502, 503, 505, 504, 506, 507, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 498, 509,
	508, 0, 0, 445, 446, 447, 448, 449, 450, 451,
	452, 453, 456, 457, 458, 486, 487, 437, 438, 439,
	440, 441, 442, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 133, 0, 23, 33,
	0, 433, 34, 0, 0, 24, 0, 29, 86, 87,
	0, 0, 0, 0, 0, 0, 106, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 0, 0,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 104,
	73, 14, 0, 32, 776, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 88, 89, 70,
	47, 93, 94, 36, 0, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	133, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 0, 0, 0, 0, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 104, 73, 14, 0, 32, 847, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 88, 89, 70, 47, 93, 94, 36, 0, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 133, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 0, 0,
	0, 0, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 25, 0, 0, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 104, 73, 14,
	0, 32, 682, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 133, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 0, 0, 0, 0, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	0, 0, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 104, 73, 14, 0, 32, 659, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 133, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 0, 0, 0, 0,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 0, 0, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 104, 73, 14, 0, 32,
	641, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 133, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 86, 87,
	0, 0, 0, 0, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 0, 0,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 104,
	73, 14, 0, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 443, 444, 454,
	455, 0, 0, 882, 0, 0, 0, 0, 0, 0,
	0, 0, 459, 460, 461, 462, 463, 464, 465, 466,
	467, 468, 469, 489, 490, 491, 492, 493, 481, 482,
	483, 484, 485, 470, 471, 472, 473, 474, 475, 476,
	477, 478, 479, 480, 0, 501, 499, 500, 496, 497,
	0, 0, 488, 494, 495, 502, 503, 505, 504, 506,
	507, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 881, 509, 508, 112, 0, 445, 446, 447,
	448, 449, 450, 451, 452, 453, 456, 457, 458, 486,
	487, 437, 438, 439, 440, 441, 442, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 443, 444, 454, 455, 0, 0, 882, 0,
	0, 0, 0, 0, 0, 0, 900, 459, 460, 461,
	462, 463, 464, 465, 466, 467, 468, 469, 489, 490,
	491, 492, 493, 481, 482, 483, 484, 485, 470, 471,
	472, 473, 474, 475, 476, 477, 478, 479, 480, 0,
	501, 499, 500, 496, 497, 0, 0, 488, 494, 495,
	502, 503, 505, 504, 506, 507, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 881, 509, 508,
	112, 0, 445, 446, 447, 448, 449, 450, 451, 452,
	453, 456, 457, 458, 486, 487, 437, 438, 439, 440,
	441, 442, 88, 89, 70, 0, 93, 94, 117, 0,
	105, 0, 0, 0, 0, 0, 110, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 133, 0, 0, 0, 0,
	0, 872, 0, 0, 0, 0, 0, 86, 87, 0,
	0, 0, 0, 0, 0, 0, 0, 51, 109, 0,
	102, 98, 99, 100, 95, 96, 0, 0, 0, 0,
	0, 0, 103, 0, 0, 0, 0, 134, 101, 97,
	112, 534, 90, 91, 92, 0, 0, 0, 0, 85,
	53, 0, 0, 0, 74, 75, 139, 0, 0, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 104, 73,
	0, 0, 0, 0, 60, 530, 52, 133, 0, 0,
	57, 56, 58, 59, 71, 106, 0, 0, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 227,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 133, 52, 0,
	0, 226, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 534, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 133, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 594,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 426, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 133, 52, 0,
	0, 593, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 133, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 133, 52, 0,
	0, 389, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 88, 89, 70, 0, 93, 94,
	117, 0, 105, 0, 0, 0, 0, 0, 110, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 0, 0, 365, 60, 133, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 0, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 139, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	104, 73, 0, 181, 178, 0, 60, 0, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 153, 154,
	165, 168, 169, 170, 171, 172, 173, 175, 177, 0,
	155, 157, 156, 179, 0, 0, 0, 0, 756, 180,
	159, 163, 162, 0, 0, 0, 0, 0, 158, 0,
	160, 164, 166, 167, 174, 176, 161, 181, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 153, 154, 165, 168, 169, 170, 171, 172,
	173, 175, 177, 0, 155, 157, 156, 179, 0, 0,
	709, 0, 0, 180, 159, 163, 162, 0, 0, 0,
	0, 0, 158, 0, 160, 164, 166, 167, 174, 176,
	161, 181, 178, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 153, 154, 165, 168,
	169, 170, 171, 172, 173, 175, 177, 0, 0, 0,
	694, 155, 157, 156, 179, 0, 0, 180, 159, 163,
	162, 0, 0, 0, 0, 0, 158, 0, 160, 164,
	166, 167, 174, 176, 161, 0, 0, 0, 181, 178,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 153, 154, 165, 168, 169, 170, 171,
	172, 173, 175, 177, 0, 0, 0, 692, 155, 157,
	156, 179, 0, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 158, 0, 160, 164, 166, 167, 174,
	176, 161, 0, 0, 0, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	153, 154, 165, 168, 169, 170, 171, 172, 173, 175,
	177, 0, 0, 0, 683, 155, 157, 156, 179, 0,
	0, 180, 159, 163, 162, 0, 0, 0, 0, 0,
	158, 0, 160, 164, 166, 167, 174, 176, 161, 0,
	0, 0, 181, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 153, 154, 165,
	168, 169, 170, 171, 172, 173, 175, 177, 0, 155,
	157, 156, 179, 0, 0, 679, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 158, 0, 160,
	164, 166, 167, 174, 176, 161, 181, 178, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 153, 154, 165, 168, 169, 170, 171, 172, 173,
	175, 177, 0, 155, 157, 156, 179, 0, 0, 678,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 158, 0, 160, 164, 166, 167, 174, 176, 161,
	181, 178, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 153, 154, 165, 168, 169,
	170, 171, 172, 173, 175, 177, 0, 0, 0, 624,
	155, 157, 156, 179, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 158, 0, 160, 164, 166,
	167, 174, 176, 161, 0, 0, 0, 181, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 153, 154, 165, 168, 169, 170, 171, 172,
	173, 175, 177, 0, 155, 157, 156, 179, 0, 0,
	623, 0, 0, 180, 159, 163, 162, 0, 0, 0,
	0, 0, 158, 0, 160, 164, 166, 167, 174, 176,
	161, 181, 178, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 153, 154, 165, 168,
	169, 170, 171, 172, 173, 175, 177, 0, 155, 157,
	156, 179, 0, 0, 622, 0, 0, 180, 159, 163,
	162, 0, 0, 0, 0, 0, 158, 0, 160, 164,
	166, 167, 174, 176, 161, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	153, 154, 165, 168, 169, 170, 171, 172, 173, 175,
	177, 0, 0, 0, 605, 155, 157, 156, 179, 0,
	0, 180, 159, 163, 162, 0, 0, 0, 0, 0,
	158, 0, 160, 164, 166, 167, 174, 176, 161, 0,
	0, 0, 181, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 153, 154, 165,
	168, 169, 170, 171, 172, 173, 175, 177, 0, 155,
	157, 156, 179, 0, 0, 596, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 158, 0, 160,
	164, 166, 167, 174, 176, 161, 181, 178, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 153, 154, 165, 168, 169, 170, 171, 172, 173,
	175, 177, 567, 0, 0, 586, 155, 157, 156, 179,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 158, 0, 160, 164, 166, 167, 174, 176, 161,
	0, 0, 0, 181, 178, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 153, 154,
	165, 168, 169, 170, 171, 172, 173, 175, 177, 0,
	155, 157, 156, 179, 0, 0, 584, 0, 0, 180,
	159, 163, 162, 0, 0, 0, 0, 0, 158, 0,
	160, 164, 166, 167, 174, 176, 161, 181, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 153, 154, 165, 168, 169, 170, 171, 172,
	173, 175, 177, 0, 155, 157, 156, 179, 0, 0,
	0, 0, 0, 180, 159, 163, 162, 0, 0, 0,
	0, 0, 158, 0, 160, 164, 166, 167, 174, 176,
	161, 181, 178, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 153, 154, 165, 168,
	169, 170, 171, 172, 173, 175, 177, 0, 155, 157,
	156, 179, 563, 0, 0, 0, 0, 180, 159, 163,
	162, 0, 0, 0, 0, 0, 158, 0, 160, 164,
	166, 167, 174, 176, 161, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	153, 154, 165, 168, 169, 170, 171, 172, 173, 175,
	177, 0, 155, 157, 156, 179, 0, 0, 557, 0,
	0, 180, 159, 163, 162, 0, 0, 0, 0, 0,
	158, 0, 160, 164, 166, 167, 174, 176, 161, 181,
	178, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 153, 154, 165, 168, 169, 170,
	171, 172, 173, 175, 177, 0, 155, 157, 156, 179,
	0, 0, 553, 0, 0, 180, 159, 163, 162, 0,
	0, 0, 0, 0, 158, 0, 160, 164, 166, 167,
	174, 176, 161, 181, 178, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 412, 0, 0, 0, 153, 154,
	165, 168, 169, 170, 171, 172, 173, 175, 177, 0,
	0, 0, 0, 0, 0, 0, 417, 0, 0, 180,
	159, 163, 162, 155, 157, 156, 179, 0, 158, 0,
	160, 164, 166, 167, 174, 176, 161, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	181, 178, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 153, 154, 165, 168, 169,
	170, 171, 172, 173, 175, 177, 0, 155, 157, 156,
	179, 0, 0, 0, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 158, 0, 160, 164, 166,
	167, 174, 176, 161, 181, 178, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 153,
	154, 165, 168, 169, 170, 171, 172, 173, 175, 177,
	0, 0, 0, 0, 373, 155, 157, 156, 179, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 158,
	0, 160, 164, 166, 167, 174, 176, 161, 0, 0,
	0, 0, 181, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 153, 154, 165,
	168, 169, 170, 171, 172, 173, 175, 177, 0, 0,
	0, 0, 152, 155, 157, 156, 179, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 158, 0, 160,
	164, 166, 167, 174, 176, 161, 0, 0, 0, 0,
	181, 178, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 153, 154, 165, 168, 169,
	170, 171, 172, 173, 175, 177, 0, 0, 157, 156,
	179, 0, 0, 0, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 158, 0, 160, 164, 166,
	167, 174, 176, 161, 181, 178, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 153,
	154, 165, 168, 169, 170, 171, 172, 173, 175, 177,
	0, 0, 0, 156, 179, 0, 0, 0, 0, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 158,
	0, 160, 164, 166, 167, 174, 176, 161, 181, 178,
	428, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 153, 154, 165, 168, 169, 170, 171,
	172, 173, 175, 177, 0, 0, 0, 0, 0, 0,
	0, 0, 179, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 158, 0, 160, 164, 166, 167, 174,
	176, 161, 0, 0, 0, 0, 181, 178, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 153, 154, 165, 168, 169, 170, 171, 172, 173,
	175, 177, 0, 0, 0, 0, 179, 0, 0, 0,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 158, 0, 160, 164, 166, 167, 174, 176, 161,
	181, 178, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 153, 154, 165, 168, 169,
	170, 171, 172, 173, 175, 177, 0, 0, 0, 0,
	179, 0, 0, 0, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 158, 0, 160, 164, 166,
	167, 174, 176, 161, 181, 178, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 153,
	154, 165, 168, 169, 170, 171, 172, 173, 175, 177,
	0, 0, 0, 179, 0, 0, 0, 0, 0, 0,
	0, 159, 163, 162, 0, 0, 0, 0, 0, 158,
	0, 160, 164, 166, 167, 174, 176, 161, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 179, 154, 165, 168, 169, 170, 171, 172,
	173, 175, 177, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 159, 163, 162, 178, 0, 0,
	0, 0, 158, 0, 160, 164, 166, 167, 174, 176,
	161, 179, 0, 165, 168, 169, 170, 171, 172, 173,
	175, 177, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 159, 163, 162, 178, 0, 0, 0,
	0, 158, 0, 160, 164, 166, 167, 174, 176, 161,
	179, 0, 165, 168, 169, 170, 171, 172, 173, 175,
	177, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 159, 163, 162, 178, 0, 0, 0, 0,
	0, 0, 160, 164, 166, 167, 174, 176, 161, 179,
	0, 165, 168, 169, 170, 171, 172, 173, 175, 177,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 159, 163, 162, 178, 0, 0, 0, 0, 0,
	0, 0, 164, 166, 167, 174, 176, 161, 0, 0,
	165, 168, 169, 170, 171, 172, 173, 175, 177, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 163, 162, 443, 444, 454, 455, 0, 0, 434,
	0, 164, 166, 167, 174, 176, 161, 0, 459, 460,
	461, 462, 463, 464, 465, 466, 467, 468, 469, 489,
	490, 491, 492, 493, 481, 482, 483, 484, 485, 470,
	471, 472, 473, 474, 475, 476, 477, 478, 479, 480,
	0, 501, 499, 500, 496, 497, 0, 0, 488, 494,
	495, 502, 503, 505, 504, 506, 507, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 498, 509,
	508, 0, 0, 445, 446, 447, 448, 449, 450, 451,
	452, 453, 456, 457, 458, 486, 487, 437, 438, 439,
	440, 441, 442, 443, 444, 454, 455, 0, 0, 916,
	0, 0, 0, 0, 0, 0, 0, 0, 459, 460,
	461, 462, 463, 464, 465, 466, 467, 468, 469, 489,
	490, 491, 492, 493, 481, 482, 483, 484, 485, 470,
	471, 472, 473, 474, 475, 476, 477, 478, 479, 480,
	0, 501, 499, 500, 496, 497, 0, 0, 488, 494,
	495, 502, 503, 505, 504, 506, 507, 117, 0, 105,
	0, 0, 0, 0, 0, 110, 0, 0, 498, 509,
	508, 0, 0, 445, 446, 447, 448, 449, 450, 451,
	452, 453, 456, 457, 458, 486, 487, 745, 746, 747,
	744, 743, 742, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 655, 109, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 134, 0, 0, 112,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 233, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 231, 0, 656, 0, 0, 654, 0,
	0, 0, 0, 0, 106,
}
var yyPact = [...]int{

	-1000, -1000, 1241, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	224, 451, 569, 683, -1000, -1000, -1000, 217, 4449, 214,
	211, 5600, 5600, 5600, 131, 640, 5600, -1000, 6798, 209,
	207, 206, -1000, 379, 5600, 724, 232, 0, 495, 721,
	715, 713, 455, 446, 940, -1000, -1000, 205, -1000, -1000,
	126, 204, 4940, 5600, 618, 618, 5600, 5600, 5600, 5600,
	5600, -1000, -1000, 5600, 5600, 5600, 5600, 5600, 5600, 5600,
	203, 5600, -1000, 827, 5600, 5600, 5600, -1000, -1000, -1000,
	82, -1000, 503, 500, -1000, 280, 202, 200, 5600, 5600,
	192, 5600, 5600, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, 787, 799, -1000, 136, 172, 172, 190,
	-1000, 487, 662, 139, 662, 227, -1000, -1000, 314, 553,
	-13, 542, 662, -1000, -1000, -1000, -1000, -28, -1000, -58,
	3269, 5600, 594, 0, 462, 5600, 5600, 313, 6856, 567,
	311, 302, -29, -1000, -1000, -30, 0, -1000, -59, -32,
	-1000, 6856, -1000, 5600, 5600, 5600, 5600, 5600, 5600, 5600,
	5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600,
	5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600, 344,
	5490, 5600, 618, 5600, 683, -1000, 6740, 301, -1000, 710,
	-1000, 703, -1000, 532, -1000, 543, 188, 4449, 185, 300,
	216, 5380, 5600, 5600, 5600, 5600, 5600, 5600, 5600, 5600,
	5600, 5600, 5600, 5600, -1000, -1000, 5600, 5600, 5600, 95,
	4940, 74, -11, -1000, -1000, 6686, 618, 183, -1000, -1000,
	82, 5600, -1000, -1000, 4940, -1000, 403, 403, 412, 403,
	6619, 403, 403, 403, 403, 403, 403, 403, -1000, 5600,
	403, 388, 619, 793, -1000, 163, 5270, 618, 7076, 7022,
	7076, 5600, 3579, 3579, 172, -1000, 499, 191, 172, -1000,
	-1000, 5600, 5600, 6856, 6856, 5600, 6856, 6856, 705, -1000,
	695, 498, 619, 5600, -1000, -1000, 4828, -1000, 4940, 694,
	487, 299, 487, -1000, -1000, 1085, -1000, 296, -33, 537,
	662, -1000, 552, 472, 693, 531, -1000, -1000, 683, 5600,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, 182, 6565,
	180, -1000, 295, -12, 6856, 6511, -1000, -1000, -1000, -1000,
	131, -1000, 646, -1000, 5600, -1000, 5600, 7183, 7222, 6910,
	7076, 6964, 7261, 7339, 7300, 132, 132, 132, 412, 403,
	412, 412, 254, 254, 321, 321, 321, 321, 150, 150,
	150, 150, 321, -1000, 6457, 5600, 7130, -14, -1000, -1000,
	6403, -34, 3113, -1000, -1000, -1000, 179, 532, 520, 544,
	377, -1000, 544, 5600, -1000, 5600, -1000, -1000, 7076, 5600,
	7076, 7076, 7076, 7076, 7076, 7076, 7076, 7076, 7076, 7076,
	7076, 7076, 6349, 67, 6292, 172, -1000, 5600, -1000, 130,
	-65, 4940, 5160, -1000, 4940, 6238, 66, -1000, 128, -1000,
	-1000, -1000, -1000, 215, 665, 6181, 52, 334, 5600, 43,
	172, -1000, -1000, 5600, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	172, -1000, -1000, -1000, -1000, 131, 5600, 5600, 95, 131,
	532, -18, -1000, 6856, 6127, 6073, -1000, -1000, -1000, 6016,
	-1000, -19, -1000, 6856, 5600, 124, -1000, -1000, 929, -1000,
	-1000, -1000, 461, 517, -1000, 662, 508, 644, -1000, 460,
	-1000, 6856, 118, 4295, 5600, 5600, 5600, 177, -1000, -1000,
	170, 6856, -1000, 5600, 7130, 113, 618, 7647, 4141, -1000,
	162, 471, 520, -1000, 544, -1000, -1000, 374, -40, -1000,
	5962, 5908, 2957, 7339, 3987, -1000, -1000, -1000, 5851, -73,
	5600, -1000, 6856, 618, 161, 112, -1000, -1000, -1000, 41,
	-1000, -1000, 607, -1000, -1000, -1000, -1000, 5600, -1000, 7076,
	-1000, -1000, 5794, -1000, -1000, 40, 5737, -1000, -1000, 520,
	111, 5600, -1000, -1000, -1000, 108, 5050, 6856, -1000, -1000,
	662, 454, -31, -1000, -1000, 662, 644, -1000, 294, -1000,
	-1000, -1000, 5683, 292, 6856, -1000, 291, 289, 471, 7130,
	288, -1000, 107, 515, 618, 159, 4940, -1000, -1000, -1000,
	591, 471, 106, -23, -1000, 2, -1000, -1000, 677, -1000,
	-1000, -1000, -1000, 367, -40, 1421, -1000, 544, 4449, 178,
	287, -1000, -1000, -1000, 5600, 7076, -1000, 4940, -73, -1000,
	-1000, 5629, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-38, -1000, 662, 326, 644, -1000, -31, -1000, 2801, 286,
	5600, 395, -1000, 696, -1000, 104, -1000, 3679, 7647, -1000,
	4940, 39, 2645, -1000, 158, 366, 102, 557, 471, 459,
	-1000, -1000, 365, -1000, -1000, -1000, 627, 633, 544, 546,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, 1265, -1000,
	-1000, -1000, -1000, 3425, 7076, 99, 325, 362, 324, 662,
	-38, -1000, -1000, 323, 276, -1000, 98, -1000, 5600, 155,
	360, 270, 659, 557, -1000, -1000, -1000, 96, -1000, 92,
	-1000, 266, 544, -1000, 97, 97, 151, -1000, 630, -1000,
	-1000, 1109, -35, -1000, -72, 7489, 0, -48, -1000, -1000,
	3425, -73, -1000, -1000, -1000, -1000, 322, -1000, -1000, 3833,
	361, -1000, -1000, -1000, -1000, -1000, 265, 97, 2489, 3679,
	-1000, -1000, 79, -1000, 2333, 347, 471, 338, 140, -74,
	953, -1000, -1000, 627, -1000, 5600, -36, -1000, -75, 7489,
	-1000, -1000, 4728, 730, -1000, -1000, -1000, -1000, -1000, 3425,
	-1000, 337, 260, -1000, 86, 544, -1000, -1000, -1000, -1000,
	-41, -1000, -1000, 613, 5600, -1000, -1000, 6856, -1000, 7489,
	5600, -1000, -1000, 4603, -1000, 253, 237, 547, 580, 494,
	-1000, 462, -1000, -1000, 2177, 3425, -1000, -1000, 336, -1000,
	2021, 1865, -1000, 140, -1000, 6856, -1000, -1000, 6856, 133,
	-1000, -1000, -1000, -1000, 544, 7589, 7489, 236, 1709, -1000,
	-1000, -1000, -1000, -1000, 471, -40, -1000, -1000, 7489, -1000,
	-1000, -1000, 1553, 84, -1000, -1000, 97, 261, -1000, -1000,
	-1000, 1397, -1000,
}
var yyPgo = [...]int{

	0, 897, 896, 51, 9, 895, 3, 29, 16, 894,
	11, 31, 79, 78, 49, 45, 893, 21, 892, 73,
	19, 55, 891, 0, 80, 890, 889, 38, 141, 25,
	887, 36, 886, 56, 62, 883, 10, 880, 879, 872,
	868, 13, 46, 866, 858, 100, 84, 199, 856, 855,
	853, 5, 852, 83, 40, 845, 138, 43, 840, 837,
	835, 825, 823, 119, 822, 819, 818, 817, 12, 815,
	814, 44, 42, 32, 2, 15, 629, 41, 74, 812,
	809, 807, 14, 806, 803, 47, 39, 802, 18, 8,
	686, 20, 536, 800, 121, 798, 797, 795, 71, 791,
	33, 789, 787, 30, 37, 786, 784, 28, 783, 776,
	545, 773, 768, 767, 26, 765, 63, 1, 4, 763,
	17, 761, 760, 755, 7, 745, 6, 742,
}
var yyR1 = [...]int{

	0, 127, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 5,
	5, 5, 5, 5, 5, 5, 6, 6, 116, 116,
	94, 94, 10, 10, 10, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	90, 90, 16, 16, 18, 18, 7, 7, 104, 104,
	103, 103, 110, 110, 17, 17, 20, 20, 19, 19,
	98, 98, 117, 117, 22, 22, 22, 22, 22, 22,
	22, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 11, 96, 96, 95, 95, 26,
	26, 109, 109, 27, 12, 1, 1, 2, 2, 13,
	13, 125, 125, 76, 76, 14, 15, 85, 85, 87,
	87, 86, 86, 91, 91, 91, 91, 83, 83, 82,
	82, 25, 25, 80, 80, 80, 80, 107, 107, 107,
	8, 8, 84, 84, 67, 67, 65, 65, 69, 69,
	66, 66, 118, 118, 119, 119, 29, 29, 30, 30,
	75, 75, 73, 73, 73, 74, 74, 77, 77, 115,
	115, 31, 31, 102, 102, 33, 106, 106, 34, 34,
	120, 120, 35, 35, 35, 35, 124, 124, 79, 79,
	79, 108, 108, 36, 36, 37, 38, 38, 38, 38,
	40, 40, 39, 81, 81, 122, 122, 121, 121, 123,
	123, 89, 89, 89, 89, 89, 89, 105, 105, 41,
	41, 97, 97, 68, 21, 99, 99, 42, 100, 100,
	101, 101, 44, 43, 43, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 126, 3, 3, 88, 88, 111, 111, 51, 51,
	52, 52, 52, 52, 45, 45, 46, 46, 49, 49,
	93, 93, 93, 78, 78, 56, 56, 56, 50, 50,
	50, 50, 50, 50, 50, 50, 50, 50, 50, 50,
	50, 50, 50, 50, 57, 57, 57, 23, 23, 24,
	24, 55, 58, 58, 58, 59, 59, 59, 60, 60,
	60, 60, 60, 60, 28, 28, 28, 47, 47, 47,
	61, 61, 62, 62, 62, 62, 62, 62, 53, 53,
	53, 54, 54, 54, 114, 71, 71, 113, 113, 70,
	70, 70, 70, 70, 70, 92, 92, 92, 92, 63,
	63, 63, 63, 63, 63, 63, 64, 64, 64, 64,
	48, 48, 48, 48, 48, 48, 48, 112, 112, 72,
}
var yyR2 = [...]int{

	0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 2, 0,
	1, 3, 1, 3, 2, 1, 1, 1, 1, 1,
	1, 4, 3, 5, 4, 3, 4, 3, 4, 3,
	1, 1, 6, 7, 6, 7, 0, 1, 3, 1,
	3, 1, 3, 1, 1, 2, 1, 3, 1, 2,
	3, 1, 2, 0, 1, 1, 1, 1, 1, 1,
	4, 3, 1, 1, 5, 7, 9, 5, 3, 3,
	3, 3, 3, 3, 1, 2, 6, 7, 9, 5,
	1, 6, 3, 3, 2, 0, 9, 1, 3, 0,
	4, 1, 3, 1, 11, 0, 1, 0, 1, 9,
	8, 1, 2, 1, 1, 6, 7, 0, 2, 0,
	2, 0, 2, 1, 2, 4, 3, 1, 4, 1,
	4, 1, 4, 3, 4, 4, 5, 0, 5, 4,
	1, 1, 1, 4, 5, 6, 1, 3, 6, 7,
	3, 6, 1, 0, 1, 3, 4, 6, 0, 1,
	1, 2, 1, 1, 1, 0, 2, 2, 4, 1,
	3, 1, 2, 3, 1, 1, 3, 1, 1, 3,
	2, 0, 3, 4, 3, 10, 1, 3, 1, 2,
	3, 1, 2, 2, 2, 3, 3, 3, 4, 3,
	1, 1, 3, 1, 3, 1, 1, 0, 1, 1,
	2, 1, 1, 1, 1, 1, 1, 3, 1, 2,
	4, 3, 1, 4, 4, 3, 1, 1, 0, 1,
	3, 1, 8, 3, 2, 6, 5, 3, 4, 2,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 2, 2, 2, 2, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 2, 2, 2, 2, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 1, 5,
	4, 3, 1, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 1, 3, 2, 1, 2, 4, 2, 11,
	12, 0, 0, 1, 0, 4, 3, 1, 1, 2,
	2, 4, 4, 2, 1, 1, 1, 1, 0, 3,
	0, 1, 1, 0, 1, 4, 3, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 3, 2,
	3, 3, 1, 1, 1, 3, 3, 1, 1, 0,
	1, 1, 1, 3, 1, 1, 3, 1, 1, 4,
	4, 4, 4, 1, 1, 1, 3, 1, 4, 2,
	3, 3, 1, 4, 4, 3, 3, 3, 1, 3,
	1, 1, 3, 1, 1, 0, 1, 3, 1, 3,
	1, 4, 2, 6, 4, 2, 2, 1, 2, 1,
	4, 3, 3, 3, 6, 3, 1, 1, 2, 1,
	5, 4, 2, 2, 4, 2, 2, 1, 3, 1,
}
var yyChk = [...]int{

	-1000, -127, -116, -9, 2, -11, -12, -13, -14, -15,
	51, 79, 44, 38, 142, -65, -66, 21, 20, 23,
	30, 34, 35, 39, 46, 98, 19, 14, -23, 48,
	25, 27, 144, 40, 43, 36, 10, 37, -125, 52,
	53, 54, -67, -69, -28, -32, -76, 7, -60, -61,
	-58, 59, 148, 92, 104, 105, 153, 152, 154, 155,
	146, -43, -48, 107, 108, 109, 110, 111, 112, 113,
	6, 156, -50, 141, 96, 97, 106, 99, 100, -47,
	-57, -52, -45, -55, -56, 91, 49, 50, 4, 5,
	84, 85, 86, 8, 9, 66, 67, 81, 63, 64,
	65, 80, 62, 74, 140, 12, 157, -10, -59, 60,
	18, -94, 82, 146, 82, -94, 142, 10, -18, -90,
	-110, -94, 82, 37, 38, -19, -20, -98, -21, 10,
	-117, 146, -11, 37, 79, 146, 146, -24, -23, 98,
	-24, -24, -102, -33, -47, -106, 37, -34, 12, -99,
	-42, -23, 144, 129, 130, 87, 89, 88, 159, 151,
	161, 167, 153, 152, 162, 131, 163, 164, 132, 133,
	134, 135, 136, 137, 165, 138, 166, 139, 115, 90,
	150, 114, 146, 146, 146, 142, -23, 10, 145, -3,
	151, 52, -76, 10, 10, 10, 93, 94, 93, 95,
	94, 160, 117, 118, 119, 120, 121, 122, 123, 124,
	125, 126, 127, 128, 104, 105, 146, 148, 142, 57,
	146, -114, -113, -71, -70, -23, 151, 59, -23, -28,
	-57, 146, -56, 98, 148, -28, -23, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -23, -49, 146,
	-23, -93, 17, -92, -63, 12, 76, 77, -23, -23,
	-23, 148, 78, 78, -46, -44, -45, -62, 52, -10,
	-47, 146, 146, -23, -23, 146, -23, -23, 17, 75,
	-92, -92, 17, 142, -47, -77, 146, -77, 146, 82,
	-94, 147, -94, 144, 142, -116, 144, -16, -110, -94,
	82, 144, 158, 82, 29, -94, -20, 144, 158, 160,
	-22, 143, 2, -11, -12, -13, -14, -15, 51, -23,
	21, -3, -100, -101, -23, -23, 144, 144, 144, 144,
	158, 144, 158, -3, 160, 144, 158, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -46, -23, 145, -23, -109, -27, -28,
	-23, -98, -117, 144, 144, 10, -126, 10, -85, 55,
	-126, -87, 55, 146, -11, 146, 144, 145, -23, 151,
	-23, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -24, -23, -54, 10, 142, -47, -114,
	149, 158, 58, -28, 146, -23, -114, 147, -24, 141,
	-63, -63, 17, 148, 57, -23, 11, -28, 58, -24,
	-53, -6, -47, 142, 10, -5, -4, 98, 99, 100,
	101, 102, 103, 4, 5, 84, 85, 86, 87, 88,
	89, 90, 91, 92, 6, 7, 93, 94, 95, 19,
	20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
	40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
	50, 35, 36, 37, 38, 39, 96, 97, 59, 30,
	31, 32, 33, 34, 60, 61, 55, 56, 79, 53,
	54, 52, 62, 63, 65, 64, 66, 67, 81, 80,
	-53, -6, -47, -78, -77, 78, 148, 142, 57, 78,
	-78, -112, -72, -23, -23, -23, 75, 75, 140, -23,
	147, -115, -31, -23, 83, -114, 10, 144, -116, 143,
	144, 144, 82, -94, -19, 82, -94, 142, 10, 82,
	-21, -23, 146, 147, 146, 144, 158, 147, -33, -34,
	-126, -23, -42, 145, -23, -7, 158, 29, 147, 143,
	-126, 146, -85, -86, 56, -10, 142, -126, -124, -10,
	-23, -23, -117, -23, 147, 149, 143, -77, -23, 147,
	160, -71, -23, 151, 59, -114, 147, 149, 147, -64,
	10, 13, 152, 12, 10, 143, 143, 148, 143, -23,
	149, -77, -23, -77, -47, -24, -23, -54, -47, -85,
	-7, 158, 147, 147, 143, -7, 158, -23, 147, 143,
	142, 82, -104, -17, -20, -90, 142, -126, 147, -84,
	-11, 145, -23, -100, -23, -80, 142, 145, 146, -23,
	147, -27, -91, -28, 151, 59, 148, -25, -11, 145,
	-96, 146, -118, -119, -29, -30, -75, -73, 150, 60,
	61, -10, -86, -126, -124, -120, 142, 158, 147, 147,
	95, -11, 145, 143, 160, -23, -28, 146, 147, 149,
	13, -23, 143, 149, 143, -86, 147, -72, 147, -31,
	-103, -20, 142, -7, 158, -20, -104, 144, -117, 147,
	144, -107, 144, -107, 144, -118, 144, 147, 58, -28,
	146, -114, -117, -26, 41, 42, -118, 147, 158, -1,
	151, -73, -126, 142, 143, -35, -122, -121, 44, -123,
	47, -89, 103, 102, 101, 98, 99, 100, -120, -10,
	-11, 145, 144, -117, -23, -114, 149, -126, -7, 158,
	-103, 143, -17, -7, 22, 144, -100, 143, 32, 33,
	-107, 31, -107, 147, -82, -11, 145, -91, -28, -114,
	149, 28, 146, 142, 147, -88, 44, -29, -2, 83,
	142, -120, -105, -41, 12, 38, 37, -124, -89, 143,
	-117, 147, 143, 142, 143, -20, -7, 143, 144, 147,
	-23, -8, 145, 144, 143, 144, 31, -88, -117, 147,
	147, 144, -95, -10, -117, -74, 145, -74, 146, 12,
	-120, 143, 144, 158, -126, 160, -97, -68, -6, -3,
	-79, 144, 142, -120, 143, -83, -11, 145, -8, -117,
	144, -74, 26, -82, 12, 159, 143, 142, -75, 142,
	-111, -51, 12, 151, 160, 143, -41, -23, 144, 158,
	160, -6, 143, -108, -36, -37, -38, -39, -40, -10,
	-6, 79, 10, 143, -117, -117, 142, 144, 147, -10,
	-117, -117, 147, 158, 12, -23, -126, -68, -23, -126,
	143, -36, 144, 144, 45, 29, 78, 24, -117, 142,
	143, 143, -51, -126, 146, -124, 10, -4, -89, -6,
	144, 143, -117, -118, -6, 143, 147, -74, -81, 144,
	142, -117, 143,
}
var yyDef = [...]int{

	79, -2, -2, 78, 85, 86, 87, 88, 89, 90,
	0, 0, 0, 0, 123, 132, 133, 0, 0, 0,
	0, 419, 419, 419, 0, 384, 0, 144, 0, 0,
	0, 0, 150, 0, 0, 0, 80, 372, 0, 0,
	0, 0, 206, 0, -2, 418, 171, 0, -2, 435,
	421, 0, 455, 0, 0, 0, 0, 0, 0, 0,
	0, 348, 352, 0, 0, 0, 0, 0, 0, 0,
	388, 0, 362, 390, 0, 365, 0, 173, 174, 428,
	413, 433, 0, 0, -2, 0, 0, 0, 0, 0,
	0, 0, 0, 398, 399, 400, 401, 402, 403, 404,
	405, 406, 407, 0, 0, 437, 0, -2, 0, 0,
	397, 82, 0, 0, 0, 0, 79, 80, 0, 0,
	0, 116, 0, 100, 101, 113, 118, 0, 121, 0,
	0, 0, 0, 372, 0, 288, 0, 0, 420, 384,
	0, 0, 0, 234, 235, 0, 372, 237, 238, 0,
	286, 287, 145, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 123, 0, 0, 154, 371,
	373, 0, 172, 177, 371, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 312, 314, 0, 419, 0, 0,
	455, 0, 454, 458, 456, 460, 0, 0, 299, -2,
	0, 0, -2, 384, 455, -2, 333, 334, 335, 336,
	0, 353, 354, 355, 356, 357, 358, 359, 360, 419,
	361, 0, 391, 392, 467, 469, 0, 0, 364, 366,
	368, 419, 0, 0, 393, 294, 386, 387, 393, 385,
	442, 0, 0, 482, 483, 0, 485, 486, 0, 409,
	0, 0, 0, 0, 439, 380, 0, 383, 455, 0,
	84, 0, 83, 92, 79, 0, 95, 0, 0, 116,
	0, 97, 0, 0, 0, 116, 119, 99, 0, 0,
	122, 131, 124, 125, 126, 127, 128, 129, 0, 0,
	0, 371, 0, 289, 291, 0, 138, 139, 140, 141,
	0, 142, 0, 371, 0, 143, 0, 316, 317, 318,
	319, 320, 321, 322, 323, 324, 325, 326, 327, 328,
	329, 330, 331, 332, -2, -2, -2, -2, -2, -2,
	-2, -2, -2, 346, 0, 0, 351, 106, 161, -2,
	0, 0, 0, 152, 153, 371, 0, 177, 181, 0,
	0, 371, 0, 0, 207, 0, 210, 123, 297, 0,
	300, 301, 302, 303, 304, 305, 306, 307, 308, 309,
	310, 311, 0, 0, 0, 436, 451, 0, 453, 0,
	396, 455, 0, -2, 455, 0, 0, -2, 0, 363,
	468, 465, 466, 0, 0, 0, 0, 422, 0, 0,
	0, -2, -2, 0, 76, 77, 69, 70, 71, 72,
	73, 74, 75, 2, 3, 4, 5, 6, 7, 8,
	9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
	19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
	29, 30, 31, 32, 33, 34, 35, 36, 37, 38,
	39, 40, 41, 42, 43, 44, 45, 46, 47, 48,
	49, 50, 51, 52, 53, 54, 55, 56, 57, 58,
	59, 60, 61, 62, 63, 64, 65, 66, 67, 68,
	0, -2, -2, 293, 394, 0, 419, 0, 0, 0,
	177, 106, 487, 489, 0, 0, 408, 411, 410, 0,
	227, 106, 229, 231, 0, 0, 81, 91, 0, 94,
	96, 98, 0, 116, 112, 0, 116, 0, 117, 0,
	120, 371, 0, 0, 0, 288, 0, 0, 233, 236,
	0, 239, 285, 0, 350, 0, 107, 0, 0, 155,
	0, -2, 181, 371, 0, 178, 241, 0, 180, 246,
	0, 0, 0, 298, 0, 429, 431, 432, 0, 0,
	0, 457, 459, 0, 0, 0, -2, 396, 389, 0,
	476, 477, 0, 479, 471, 472, 473, 0, 475, 367,
	430, 381, 0, 382, 446, 0, 0, 445, 447, 181,
	0, 107, 481, 484, 438, 0, 107, 232, 395, 93,
	0, 0, 106, 109, 114, 0, 0, 284, 0, 134,
	202, 123, 0, 0, 290, 137, 197, 197, -2, 349,
	0, 162, 0, -2, 0, 0, 455, 149, 191, 123,
	159, -2, 0, 212, 214, 165, 219, 220, 0, 222,
	223, 224, 371, 0, 182, 267, 241, 0, 0, 0,
	0, 204, 123, 452, 0, 296, -2, 455, 464, 470,
	478, 0, 449, 443, 444, 371, 480, 488, 228, 230,
	106, 111, 0, 0, 107, 115, 106, 130, 0, 0,
	288, 0, 197, 0, 197, 0, 146, 0, 0, -2,
	455, 0, 0, 151, 0, 0, 0, 374, 218, 167,
	166, 221, 0, 241, 175, 240, 0, 0, 0, -2,
	266, 269, 271, 272, 273, 274, 275, 276, 267, 247,
	205, 123, 211, -2, 295, 0, 0, 0, 0, 107,
	106, 104, 108, 0, 0, 135, 0, 193, 0, 0,
	0, 0, 0, 374, 147, 189, 123, 0, -2, 0,
	-2, 0, 0, 123, 225, 225, 0, 215, 0, 168,
	241, 267, 0, 278, 371, 0, 372, 0, 270, 176,
	-2, 463, 474, 241, 102, 110, 0, 105, 203, 0,
	0, 123, 200, 201, 194, 195, 0, 225, 0, 0,
	185, 192, 0, 157, 0, 0, 0, 0, 0, 216,
	267, 170, 242, 0, 279, 0, 0, 282, 0, 0,
	244, 248, 0, 267, 103, 136, 187, 123, 123, -2,
	196, 0, 0, 148, 0, 0, 160, 123, 226, 123,
	0, 377, 378, 0, 0, 169, 277, 371, 243, 0,
	0, 371, 249, 0, 251, 0, 0, 261, 0, 0,
	260, 57, -2, 292, 0, -2, 123, 190, 0, 158,
	0, 0, 375, 0, 379, 217, 280, 281, 371, 0,
	250, 252, 253, 254, 0, 0, 0, 0, 0, 123,
	164, 369, 376, 283, -2, 255, 256, 257, 259, 262,
	188, 370, 0, 0, 258, 156, 225, 0, 245, 263,
	123, 0, 264,
}
var yyTok1 = [...]int{

	1, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 154, 140, 3, 157, 164, 151, 3,
	146, 147, 162, 153, 158, 152, 167, 163, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 145, 144,
	165, 160, 166, 150, 156, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 148, 3, 149, 161, 3, 141, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 142, 159, 143, 155,
}
var yyTok2 = [...]int{

	2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
	12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
	22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
	42, 43, 44, 45, 46, 47, 48, 49, 50, 51,
	52, 53, 54, 55, 56, 57, 58, 59, 60, 61,
	62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
	72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
	82, 83, 84, 85, 86, 87, 88, 89, 90, 91,
	92, 93, 94, 95, 96, 97, 98, 99, 100, 101,
	102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
	112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
	122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
	132, 133, 134, 135, 136, 137, 138, 139,
}
var yyTok3 = [...]int{
	0,
}

var yyErrorMessages = [...]struct {
	state int
	token int
	msg   string
}{}

//line yaccpar:1

/*	parser for yacc output	*/

var (
	yyDebug        = 0
	yyErrorVerbose = true
)

type yyLexer interface {
	Lex(lval *yySymType) int
	Error(s string)
}

type yyParser interface {
	Parse(yyLexer) int
	Lookahead() int
}

type yyParserImpl struct {
	lval  yySymType
	stack [yyInitialStackSize]yySymType
	char  int
}

func (p *yyParserImpl) Lookahead() int {
	return p.char
}

func yyNewParser() yyParser {
	return &yyParserImpl{}
}

const yyFlag = -1000

func yyTokname(c int) string {
	if c >= 1 && c-1 < len(yyToknames) {
		if yyToknames[c-1] != "" {
			return yyToknames[c-1]
		}
	}
	return __yyfmt__.Sprintf("tok-%v", c)
}

func yyStatname(s int) string {
	if s >= 0 && s < len(yyStatenames) {
		if yyStatenames[s] != "" {
			return yyStatenames[s]
		}
	}
	return __yyfmt__.Sprintf("state-%v", s)
}

func yyErrorMessage(state, lookAhead int) string {
	const TOKSTART = 4

	if !yyErrorVerbose {
		return "syntax error"
	}

	for _, e := range yyErrorMessages {
		if e.state == state && e.token == lookAhead {
			return "syntax error: " + e.msg
		}
	}

	res := "syntax error: unexpected " + yyTokname(lookAhead)

	// To match Bison, suggest at most four expected tokens.
	expected := make([]int, 0, 4)

	// Look for shiftable tokens.
	base := yyPact[state]
	for tok := TOKSTART; tok-1 < len(yyToknames); tok++ {
		if n := base + tok; n >= 0 && n < yyLast && yyChk[yyAct[n]] == tok {
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}
	}

	if yyDef[state] == -2 {
		i := 0
		for yyExca[i] != -1 || yyExca[i+1] != state {
			i += 2
		}

		// Look for tokens that we accept or reduce.
		for i += 2; yyExca[i] >= 0; i += 2 {
			tok := yyExca[i]
			if tok < TOKSTART || yyExca[i+1] == 0 {
				continue
			}
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}

		// If the default action is to accept or reduce, give up.
		if yyExca[i+1] != 0 {
			return res
		}
	}

	for i, tok := range expected {
		if i == 0 {
			res += ", expecting "
		} else {
			res += " or "
		}
		res += yyTokname(tok)
	}
	return res
}

func yylex1(lex yyLexer, lval *yySymType) (char, token int) {
	token = 0
	char = lex.Lex(lval)
	if char <= 0 {
		token = yyTok1[0]
		goto out
	}
	if char < len(yyTok1) {
		token = yyTok1[char]
		goto out
	}
	if char >= yyPrivate {
		if char < yyPrivate+len(yyTok2) {
			token = yyTok2[char-yyPrivate]
			goto out
		}
	}
	for i := 0; i < len(yyTok3); i += 2 {
		token = yyTok3[i+0]
		if token == char {
			token = yyTok3[i+1]
			goto out
		}
	}

out:
	if token == 0 {
		token = yyTok2[1] /* unknown char */
	}
	if yyDebug >= 3 {
		__yyfmt__.Printf("lex %s(%d)\n", yyTokname(token), uint(char))
	}
	return char, token
}

func yyParse(yylex yyLexer) int {
	return yyNewParser().Parse(yylex)
}

func (yyrcvr *yyParserImpl) Parse(yylex yyLexer) int {
	var yyn int
	var yyVAL yySymType
	var yyDollar []yySymType
	_ = yyDollar // silence set and not used
	yyS := yyrcvr.stack[:]

	Nerrs := 0   /* number of errors */
	Errflag := 0 /* error recovery flag */
	yystate := 0
	yyrcvr.char = -1
	yytoken := -1 // yyrcvr.char translated into internal numbering
	defer func() {
		// Make sure we report no lookahead when not parsing.
		yystate = -1
		yyrcvr.char = -1
		yytoken = -1
	}()
	yyp := -1
	goto yystack

ret0:
	return 0

ret1:
	return 1

yystack:
	/* put a state and value onto the stack */
	if yyDebug >= 4 {
		__yyfmt__.Printf("char %v in %v\n", yyTokname(yytoken), yyStatname(yystate))
	}

	yyp++
	if yyp >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyS[yyp] = yyVAL
	yyS[yyp].yys = yystate

yynewstate:
	yyn = yyPact[yystate]
	if yyn <= yyFlag {
		goto yydefault /* simple state */
	}
	if yyrcvr.char < 0 {
		yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
	}
	yyn += yytoken
	if yyn < 0 || yyn >= yyLast {
		goto yydefault
	}
	yyn = yyAct[yyn]
	if yyChk[yyn] == yytoken { /* valid shift */
		yyrcvr.char = -1
		yytoken = -1
		yyVAL = yyrcvr.lval
		yystate = yyn
		if Errflag > 0 {
			Errflag--
		}
		goto yystack
	}

yydefault:
	/* default state action */
	yyn = yyDef[yystate]
	if yyn == -2 {
		if yyrcvr.char < 0 {
			yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
		}

		/* look through exception table */
		xi := 0
		for {
			if yyExca[xi+0] == -1 && yyExca[xi+1] == yystate {
				break
			}
			xi += 2
		}
		for xi += 2; ; xi += 2 {
			yyn = yyExca[xi+0]
			if yyn < 0 || yyn == yytoken {
				break
			}
		}
		yyn = yyExca[xi+1]
		if yyn < 0 {
			goto ret0
		}
	}
	if yyn == 0 {
		/* error ... attempt to resume parsing */
		switch Errflag {
		case 0: /* brand new error */
			yylex.Error(yyErrorMessage(yystate, yytoken))
			Nerrs++
			if yyDebug >= 1 {
				__yyfmt__.Printf("%s", yyStatname(yystate))
				__yyfmt__.Printf(" saw %s\n", yyTokname(yytoken))
			}
			fallthrough

		case 1, 2: /* incompletely recovered error ... try again */
			Errflag = 3

			/* find a state where "error" is a legal shift action */
			for yyp >= 0 {
				yyn = yyPact[yyS[yyp].yys] + yyErrCode
				if yyn >= 0 && yyn < yyLast {
					yystate = yyAct[yyn] /* simulate a shift of "error" */
					if yyChk[yystate] == yyErrCode {
						goto yystack
					}
				}

				/* the current p has no shift on "error", pop stack */
				if yyDebug >= 2 {
					__yyfmt__.Printf("error recovery pops state %d\n", yyS[yyp].yys)
				}
				yyp--
			}
			/* there is no state on the stack with an error shift ... abort */
			goto ret1

		case 3: /* no shift yet; clobber input char */
			if yyDebug >= 2 {
				__yyfmt__.Printf("error recovery discards %s\n", yyTokname(yytoken))
			}
			if yytoken == yyEofCode {
				goto ret1
			}
			yyrcvr.char = -1
			yytoken = -1
			goto yynewstate /* try again in the same state */
		}
	}

	/* reduction by production yyn */
	if yyDebug >= 2 {
		__yyfmt__.Printf("reduce %v in:\n\t%v\n", yyn, yyStatname(yystate))
	}

	yynt := yyn
	yypt := yyp
	_ = yypt // guard against "declared and not used"

	yyp -= yyR2[yyn]
	// yyp is now the index of $0. Perform the default action. Iff the
	// reduced production is ε, $1 is possibly out of range.
	if yyp+1 >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyVAL = yyS[yyp+1]

	/* consult goto table to find next state */
	yyn = yyR1[yyn]
	yyg := yyPgo[yyn]
	yyj := yyg + yyS[yyp].yys + 1

	if yyj >= yyLast {
		yystate = yyAct[yyg]
	} else {
		yystate = yyAct[yyj]
		if yyChk[yystate] != -yyn {
			yystate = yyAct[yyg]
		}
	}
	// dummy call; replaced with literal code
	switch yynt {

	case 1:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:300
		{
			yylex.(*Parser).rootNode = node.NewRoot(yyDollar[1].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yylex.(*Parser).rootNode, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 2:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 3:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 4:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 5:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 6:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 7:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 8:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 9:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:311
		{
			yyVAL.token = yyDollar[1].token
		}
	case 10:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 11:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 12:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 13:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 14:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 15:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 16:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 17:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 18:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 19:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 20:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 21:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.token = yyDollar[1].token
		}
	case 22:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 23:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 24:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 25:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 26:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 27:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 28:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 29:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 30:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 31:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:313
		{
			yyVAL.token = yyDollar[1].token
		}
	case 32:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 33:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 34:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 35:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 36:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 37:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 38:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 39:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 40:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 41:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:314
		{
			yyVAL.token = yyDollar[1].token
		}
	case 42:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 43:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 44:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 45:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 46:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 47:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 48:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 49:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 50:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 51:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 52:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:315
		{
			yyVAL.token = yyDollar[1].token
		}
	case 53:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 54:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 55:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 56:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 57:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 58:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 59:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 60:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 61:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 62:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 63:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 64:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 65:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 66:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 67:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 68:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 69:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 70:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 71:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 72:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 73:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 74:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 75:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:325
		{
			yyVAL.token = yyDollar[1].token
		}
	case 76:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 77:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:334
		{
			yyVAL.token = yyDollar[1].token
		}
	case 78:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:341
		{
			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 79:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:349
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 80:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:358
		{
			namePart := name.NewNamePart(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{namePart}

			// save position
			yylex.(*Parser).positions.AddPosition(namePart, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(namePart, yyDollar[1].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 81:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:371
		{
			namePart := name.NewNamePart(yyDollar[3].token.Value)
			yyVAL.list = append(yyDollar[1].list, namePart)

			// save position
			yylex.(*Parser).positions.AddPosition(namePart, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(namePart, yyDollar[3].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 82:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:388
		{
			yyVAL.node = name.NewName(yyDollar[1].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 83:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:397
		{
			yyVAL.node = name.NewRelative(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[3].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NamespaceToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.NsSeparatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 84:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:410
		{
			yyVAL.node = name.NewFullyQualified(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NsSeparatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 85:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:425
		{
			// error
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 86:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:432
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 87:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:438
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 88:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:444
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 89:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:450
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 90:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:456
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 91:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:462
		{
			yyVAL.node = stmt.NewHaltCompiler()

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.HaltCompilerToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 92:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:477
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewNamespace(name, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NamespaceToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 93:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:492
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewNamespace(name, yyDollar[4].list)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NamespaceToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 94:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:508
		{
			yyVAL.node = stmt.NewNamespace(nil, yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NamespaceToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 95:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:522
		{
			yyVAL.node = yyDollar[2].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UseToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 96:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:535
		{
			yyVAL.node = yyDollar[3].node.(*stmt.GroupUse).SetUseType(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UseToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 97:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:548
		{
			yyVAL.node = stmt.NewUseList(nil, yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UseToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 98:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:561
		{
			yyVAL.node = stmt.NewUseList(yyDollar[2].node, yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UseToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 99:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:574
		{
			yyVAL.node = stmt.NewConstList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ConstToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 100:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:590
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FunctionToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 101:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:602
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ConstToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 102:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:617
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.OpenCurlyBracesToken)
			if yyDollar[5].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 103:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:636
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.OpenCurlyBracesToken)
			if yyDollar[6].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 104:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:659
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.OpenCurlyBracesToken)
			if yyDollar[5].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 105:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:678
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.NsSeparatorToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.OpenCurlyBracesToken)
			if yyDollar[6].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 106:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:701
		{
			yyVAL.token = nil
		}
	case 107:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:705
		{
			yyVAL.token = yyDollar[1].token
		}
	case 108:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:712
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 109:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:721
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 110:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:730
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 111:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:739
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 112:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:748
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 113:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:757
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 114:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:766
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 115:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:772
		{
			yyVAL.node = yyDollar[2].node.(*stmt.Use).SetUseType(yyDollar[1].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 116:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:781
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewUse(nil, name, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 117:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:792
		{
			name := name.NewName(yyDollar[1].list)
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewUse(nil, name, alias)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(alias, yyDollar[3].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 118:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:812
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 119:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:818
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NsSeparatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 120:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:830
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 121:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:839
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 122:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:848
		{
			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 123:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:856
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 124:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:865
		{
			// error
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 125:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:872
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 126:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:878
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 127:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:884
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 128:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:890
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 129:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:896
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 130:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:902
		{
			yyVAL.node = stmt.NewHaltCompiler()

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.HaltCompilerToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 131:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:919
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 132:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:932
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 133:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:938
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 134:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:944
		{
			switch n := yyDollar[5].node.(type) {
			case *stmt.While:
				n.Cond = yyDollar[3].node
			case *stmt.AltWhile:
				n.Cond = yyDollar[3].node
			}

			yyVAL.node = yyDollar[5].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.WhileToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 135:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:965
		{
			yyVAL.node = stmt.NewDo(yyDollar[2].node, yyDollar[5].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DoToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.WhileToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 136:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:981
		{
			switch n := yyDollar[9].node.(type) {
			case *stmt.For:
				n.Init = yyDollar[3].list
				n.Cond = yyDollar[5].list
				n.Loop = yyDollar[7].list
			case *stmt.AltFor:
				n.Init = yyDollar[3].list
				n.Cond = yyDollar[5].list
				n.Loop = yyDollar[7].list
			}

			yyVAL.node = yyDollar[9].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ForToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.ForInitSemicolonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.ForCondSemicolonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[8].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 137:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1008
		{
			switch n := yyDollar[5].node.(type) {
			case *stmt.Switch:
				n.Cond = yyDollar[3].node
			case *stmt.AltSwitch:
				n.Cond = yyDollar[3].node
			default:
				panic("unexpected node type")
			}

			yyVAL.node = yyDollar[5].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.SwitchToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 138:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1031
		{
			yyVAL.node = stmt.NewBreak(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.BreakToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 139:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1044
		{
			yyVAL.node = stmt.NewContinue(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ContinueToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 140:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1057
		{
			yyVAL.node = stmt.NewReturn(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ReturnToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 141:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1070
		{
			yyVAL.node = stmt.NewGlobal(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.GlobalToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 142:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1083
		{
			yyVAL.node = stmt.NewStatic(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StaticToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 143:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1096
		{
			yyVAL.node = stmt.NewEcho(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.EchoToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 144:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1109
		{
			yyVAL.node = stmt.NewInlineHtml(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.InlineHTMLToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 145:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1121
		{
			yyVAL.node = stmt.NewExpression(yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 146:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1133
		{
			yyVAL.node = stmt.NewUnset(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UnsetToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			if yyDollar[4].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 147:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:1151
		{
			switch n := yyDollar[7].node.(type) {
			case *stmt.Foreach:
				n.Expr = yyDollar[3].node
				n.Variable = yyDollar[5].node
			case *stmt.AltForeach:
				n.Expr = yyDollar[3].node
				n.Variable = yyDollar[5].node
			}

			yyVAL.node = yyDollar[7].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[7].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ForeachToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 148:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:1175
		{
			switch n := yyDollar[9].node.(type) {
			case *stmt.Foreach:
				n.Expr = yyDollar[3].node
				n.Key = yyDollar[5].node
				n.Variable = yyDollar[7].node
			case *stmt.AltForeach:
				n.Expr = yyDollar[3].node
				n.Key = yyDollar[5].node
				n.Variable = yyDollar[7].node
			}

			yyVAL.node = yyDollar[9].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ForeachToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.DoubleArrowToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[8].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 149:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1202
		{
			yyVAL.node = stmt.NewDeclare(yyDollar[3].list, yyDollar[5].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DeclareToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 150:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1216
		{
			yyVAL.node = stmt.NewNop()

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 151:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1228
		{
			if yyDollar[6].node == nil {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[5].list))
			} else {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))
			}

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.TryToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 152:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1245
		{
			yyVAL.node = stmt.NewThrow(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ThrowToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 153:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1258
		{
			label := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewGoto(label)

			// save position
			yylex.(*Parser).positions.AddPosition(label, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.GotoToken)
			yylex.(*Parser).comments.AddFromToken(label, yyDollar[2].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 154:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1274
		{
			label := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewLabel(label)

			// save position
			yylex.(*Parser).positions.AddPosition(label, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(label, yyDollar[1].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 155:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1291
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 156:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:1297
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[5].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			catch := stmt.NewCatch(yyDollar[4].list, variable, yyDollar[8].list)
			yyVAL.list = append(yyDollar[1].list, catch)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			yylex.(*Parser).positions.AddPosition(catch, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[2].token, yyDollar[9].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(catch, yyDollar[2].token, comment.CatchToken)
			yylex.(*Parser).comments.AddFromToken(catch, yyDollar[3].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[5].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(catch, yyDollar[6].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(catch, yyDollar[7].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(catch, yyDollar[9].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 157:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1321
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 158:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1327
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.VerticalBarToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 159:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1339
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 160:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1345
		{
			yyVAL.node = stmt.NewFinally(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FinallyToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 161:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1362
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 162:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1368
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 163:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1380
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 164:
		yyDollar = yyS[yypt-11 : yypt+1]
		//line php7/php7.y:1389
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewFunction(name, yyDollar[2].token != nil, yyDollar[6].list, yyDollar[8].node, yyDollar[10].list, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FunctionToken)
			if yyDollar[2].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AmpersandToken)
			}
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[3].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[9].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[11].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 165:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1415
		{
			yyVAL.token = nil
		}
	case 166:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1419
		{
			yyVAL.token = yyDollar[1].token
		}
	case 167:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1426
		{
			yyVAL.token = nil
		}
	case 168:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1430
		{
			yyVAL.token = yyDollar[1].token
		}
	case 169:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:1437
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewClass(name, yyDollar[1].list, nil, yyDollar[4].ClassExtends, yyDollar[5].ClassImplements, yyDollar[8].list, yyDollar[6].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[9].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ClassToken)
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[3].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[9].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 170:
		yyDollar = yyS[yypt-8 : yypt+1]
		//line php7/php7.y:1454
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewClass(name, nil, nil, yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ClassToken)
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[2].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[8].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 171:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1474
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 172:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1480
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 173:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1489
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.AbstractToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 174:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1501
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FinalToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 175:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1516
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewTrait(name, yyDollar[5].list, yyDollar[3].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.TraitToken)
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[2].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 176:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:1536
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewInterface(name, yyDollar[3].InterfaceExtends, yyDollar[6].list, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.InterfaceToken)
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[2].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 177:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1556
		{
			yyVAL.ClassExtends = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 178:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1562
		{
			yyVAL.ClassExtends = stmt.NewClassExtends(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.ClassExtends, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.ClassExtends, yyDollar[1].token, comment.ExtendsToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 179:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1577
		{
			yyVAL.InterfaceExtends = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 180:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1583
		{
			yyVAL.InterfaceExtends = stmt.NewInterfaceExtends(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.InterfaceExtends, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.InterfaceExtends, yyDollar[1].token, comment.ExtendsToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 181:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1598
		{
			yyVAL.ClassImplements = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 182:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1604
		{
			yyVAL.ClassImplements = stmt.NewClassImplements(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.ClassImplements, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.ClassImplements, yyDollar[1].token, comment.ImplementsToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 183:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1619
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 184:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1625
		{
			yyVAL.node = expr.NewReference(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyDollar[2].node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.AmpersandToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 185:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1637
		{
			yyVAL.node = expr.NewList(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ListToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 186:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1651
		{
			yyVAL.node = expr.NewShortList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 187:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1667
		{
			yyVAL.node = stmt.NewFor(nil, nil, nil, yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 188:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1676
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltFor(nil, nil, nil, stmtList)

			// save position
			yylex.(*Parser).positions.AddPosition(stmtList, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EndforToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 189:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1695
		{
			yyVAL.node = stmt.NewForeach(nil, nil, nil, yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 190:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1704
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltForeach(nil, nil, nil, stmtList)

			// save position
			yylex.(*Parser).positions.AddPosition(stmtList, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EndforeachToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 191:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1723
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 192:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1729
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EnddeclareToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 193:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1746
		{
			caseList := stmt.NewCaseList(yyDollar[2].list)
			yyVAL.node = stmt.NewSwitch(nil, caseList)

			// save position
			yylex.(*Parser).positions.AddPosition(caseList, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 194:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1761
		{
			caseList := stmt.NewCaseList(yyDollar[3].list)
			yyVAL.node = stmt.NewSwitch(nil, caseList)

			// save position
			yylex.(*Parser).positions.AddPosition(caseList, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[2].token, comment.SemiColonToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 195:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1777
		{
			caseList := stmt.NewCaseList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltSwitch(nil, caseList)

			// save position
			yylex.(*Parser).positions.AddPosition(caseList, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[3].token, comment.EndswitchToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 196:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1793
		{

			caseList := stmt.NewCaseList(yyDollar[3].list)
			yyVAL.node = stmt.NewAltSwitch(nil, caseList)

			// save position
			yylex.(*Parser).positions.AddPosition(caseList, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[3].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[2].token, comment.SemiColonToken)
			yylex.(*Parser).comments.AddFromToken(caseList, yyDollar[4].token, comment.EndswitchToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 197:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1814
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 198:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1820
		{
			_case := stmt.NewCase(yyDollar[3].node, yyDollar[5].list)
			yyVAL.list = append(yyDollar[1].list, _case)

			// save position
			yylex.(*Parser).positions.AddPosition(_case, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[5].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(_case, yyDollar[2].token, comment.CaseToken)
			yylex.(*Parser).comments.AddFromToken(_case, yyDollar[4].token, comment.CaseSeparatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 199:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1834
		{
			_default := stmt.NewDefault(yyDollar[4].list)
			yyVAL.list = append(yyDollar[1].list, _default)

			// save position
			yylex.(*Parser).positions.AddPosition(_default, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(_default, yyDollar[2].token, comment.DefaultToken)
			yylex.(*Parser).comments.AddFromToken(_default, yyDollar[3].token, comment.CaseSeparatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 200:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1851
		{
			yyVAL.token = yyDollar[1].token
		}
	case 201:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1855
		{
			yyVAL.token = yyDollar[1].token
		}
	case 202:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1862
		{
			yyVAL.node = stmt.NewWhile(nil, yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 203:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1871
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltWhile(nil, stmtList)

			// save position
			yylex.(*Parser).positions.AddPosition(stmtList, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EndwhileToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 204:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1890
		{
			yyVAL.node = stmt.NewIf(yyDollar[3].node, yyDollar[5].node, nil, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IfToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 205:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1904
		{
			_elseIf := stmt.NewElseIf(yyDollar[4].node, yyDollar[6].node)
			yyVAL.node = yyDollar[1].node.(*stmt.If).AddElseIf(_elseIf)

			// save position
			yylex.(*Parser).positions.AddPosition(_elseIf, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[6].node))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[2].token, comment.ElseifToken)
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[3].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[5].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 206:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1923
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 207:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1929
		{
			_else := stmt.NewElse(yyDollar[3].node)
			yyVAL.node = yyDollar[1].node.(*stmt.If).SetElse(_else)

			// save position
			yylex.(*Parser).positions.AddPosition(_else, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[3].node))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ElseToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 208:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1946
		{
			stmts := stmt.NewStmtList(yyDollar[6].list)
			yyVAL.node = stmt.NewAltIf(yyDollar[3].node, stmts, nil, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[6].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[6].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IfToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 209:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:1963
		{
			stmts := stmt.NewStmtList(yyDollar[7].list)
			_elseIf := stmt.NewAltElseIf(yyDollar[4].node, stmts)
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).AddElseIf(_elseIf)

			// save position
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[7].list))
			yylex.(*Parser).positions.AddPosition(_elseIf, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[7].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[2].token, comment.ElseifToken)
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[3].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[5].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(_elseIf, yyDollar[6].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 210:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1984
		{
			yyVAL.node = yyDollar[1].node

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EndifToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 211:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1997
		{
			stmts := stmt.NewStmtList(yyDollar[4].list)
			_else := stmt.NewAltElse(stmts)
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).SetElse(_else)

			// save position
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[4].list))
			yylex.(*Parser).positions.AddPosition(_else, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(_else, yyDollar[2].token, comment.ElseToken)
			yylex.(*Parser).comments.AddFromToken(_else, yyDollar[3].token, comment.ColonToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.EndifToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 212:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2019
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 213:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2025
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 214:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2034
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 215:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2040
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 216:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2052
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[4].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].token != nil, yyDollar[3].token != nil)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].node != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			} else if yyDollar[2].token != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[2].token, yyDollar[4].token))
			} else if yyDollar[3].token != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[3].token, yyDollar[4].token))
			} else {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			}

			// save comments
			if yyDollar[2].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AmpersandToken)
			}
			if yyDollar[3].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EllipsisToken)
			}
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[4].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 217:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:2082
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[4].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].token != nil, yyDollar[3].token != nil)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].node != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))
			} else if yyDollar[2].token != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[6].node))
			} else if yyDollar[3].token != nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[3].token, yyDollar[6].node))
			} else {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[4].token, yyDollar[6].node))
			}

			// save comments
			if yyDollar[2].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AmpersandToken)
			}
			if yyDollar[3].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.EllipsisToken)
			}
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[4].token, comment.VariableToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 218:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2116
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 219:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2122
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 220:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2131
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 221:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2137
		{
			yyVAL.node = node.NewNullable(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.QuestionMarkToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 222:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2152
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ArrayToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 223:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2164
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.CallableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 224:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2176
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 225:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2185
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 226:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2191
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 227:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2203
		{
			yyVAL.node = node.NewArgumentList(nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 228:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2216
		{
			yyVAL.node = node.NewArgumentList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			if yyDollar[3].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 229:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2235
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 230:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2241
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 231:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2253
		{
			yyVAL.node = node.NewArgument(yyDollar[1].node, false, false)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 232:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2262
		{
			yyVAL.node = node.NewArgument(yyDollar[2].node, true, false)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.EllipsisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 233:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2277
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 234:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2286
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 235:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2295
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 236:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2304
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 237:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2313
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 238:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2322
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewStaticVar(variable, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 239:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2338
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewStaticVar(variable, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 240:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2358
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 241:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2364
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 242:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2373
		{
			yyVAL.node = stmt.NewPropertyList(yyDollar[1].list, yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 243:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2385
		{
			yyVAL.node = stmt.NewClassConstList(yyDollar[1].list, yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ConstToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 244:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2398
		{
			var adaptationList *stmt.TraitAdaptationList

			switch n := yyDollar[3].node.(type) {
			case *stmt.TraitAdaptationList:
				adaptationList = n
			default:
				adaptationList = nil
				yylex.(*Parser).comments.AddFromChildNode(yyVAL.node, yyDollar[3].node)
			}

			yyVAL.node = stmt.NewTraitUse(yyDollar[2].list, adaptationList)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UseToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 245:
		yyDollar = yyS[yypt-10 : yypt+1]
		//line php7/php7.y:2420
		{
			name := node.NewIdentifier(yyDollar[4].token.Value)
			yyVAL.node = stmt.NewClassMethod(name, yyDollar[1].list, yyDollar[3].token != nil, yyDollar[7].list, yyDollar[9].node, yyDollar[10].node, yyDollar[5].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].list == nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[10].node))
			} else {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListNodePosition(yyDollar[1].list, yyDollar[10].node))
			}

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.FunctionToken)
			if yyDollar[3].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.AmpersandToken)
			}
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[4].token, comment.IdentifierToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[8].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 246:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2447
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 247:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2453
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 248:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2465
		{
			yyVAL.node = stmt.NewNop()

			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 249:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2477
		{
			yyVAL.node = stmt.NewTraitAdaptationList(nil)

			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 250:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2489
		{
			yyVAL.node = stmt.NewTraitAdaptationList(yyDollar[2].list)

			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 251:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2504
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 252:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2510
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 253:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2519
		{
			yyVAL.node = yyDollar[1].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 254:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2528
		{
			yyVAL.node = yyDollar[1].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 255:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2540
		{
			yyVAL.node = stmt.NewTraitUsePrecedence(yyDollar[1].node, yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeNodeListPosition(yyDollar[1].node, yyDollar[3].list))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.InsteadofToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 256:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2555
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)

			// save position
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(alias, yyDollar[3].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 257:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2570
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)

			// save position
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(alias, yyDollar[3].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 258:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2585
		{
			alias := node.NewIdentifier(yyDollar[4].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, alias)

			// save position
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsToken)
			yylex.(*Parser).comments.AddFromToken(alias, yyDollar[4].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 259:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2600
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 260:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2615
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewTraitMethodRef(nil, name)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[1].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 261:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2629
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 262:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2638
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitMethodRef(yyDollar[1].node, target)

			// save position
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)
			yylex.(*Parser).comments.AddFromToken(target, yyDollar[3].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 263:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2656
		{
			yyVAL.node = stmt.NewNop()

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.SemiColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 264:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2668
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 265:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2684
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 266:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2690
		{
			modifier := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{modifier}

			// save position
			yylex.(*Parser).positions.AddPosition(modifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(modifier, yyDollar[1].token, comment.VarToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 267:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2706
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 268:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2712
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 269:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2721
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 270:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2727
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 271:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2736
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.PublicToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 272:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2748
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ProtectedToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 273:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2760
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.PrivateToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 274:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2772
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StaticToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 275:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2784
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.AbstractToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 276:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2796
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FinalToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 277:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2811
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 278:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2820
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 279:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2829
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewProperty(variable, nil, yyDollar[2].str)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 280:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2845
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewProperty(variable, yyDollar[3].node, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 281:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2865
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 282:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2874
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 283:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2883
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[1].token, comment.IdentifierToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 284:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2901
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[1].token, comment.StringToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 285:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2919
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 286:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2928
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 287:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2937
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 288:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2946
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 289:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2952
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 290:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2961
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 291:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2970
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 292:
		yyDollar = yyS[yypt-8 : yypt+1]
		//line php7/php7.y:2979
		{
			if yyDollar[2].node != nil {
				yyVAL.node = stmt.NewClass(nil, nil, yyDollar[2].node.(*node.ArgumentList), yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)
			} else {
				yyVAL.node = stmt.NewClass(nil, nil, nil, yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)
			}

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ClassToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[8].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 293:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3000
		{
			if yyDollar[3].node != nil {
				yyVAL.node = expr.NewNew(yyDollar[2].node, yyDollar[3].node.(*node.ArgumentList))
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))
			} else {
				yyVAL.node = expr.NewNew(yyDollar[2].node, nil)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			}

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NewToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 294:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3015
		{
			yyVAL.node = expr.NewNew(yyDollar[2].node, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NewToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 295:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:3030
		{
			list := expr.NewList(yyDollar[3].list)
			yyVAL.node = assign.NewAssign(list, yyDollar[6].node)

			// save position
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[1].token, comment.ListToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[4].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 296:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:3047
		{
			shortList := expr.NewShortList(yyDollar[2].list)
			yyVAL.node = assign.NewAssign(shortList, yyDollar[5].node)

			// save position
			yylex.(*Parser).positions.AddPosition(shortList, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(shortList, yyDollar[1].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(shortList, yyDollar[3].token, comment.CloseSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 297:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3063
		{
			yyVAL.node = assign.NewAssign(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 298:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:3075
		{
			yyVAL.node = assign.NewReference(yyDollar[1].node, yyDollar[4].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.EqualToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.AmpersandToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 299:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3088
		{
			yyVAL.node = expr.NewClone(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.CloneToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 300:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3100
		{
			yyVAL.node = assign.NewPlus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PlusEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 301:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3112
		{
			yyVAL.node = assign.NewMinus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.MinusEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 302:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3124
		{
			yyVAL.node = assign.NewMul(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.MulEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 303:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3136
		{
			yyVAL.node = assign.NewPow(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PowEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 304:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3148
		{
			yyVAL.node = assign.NewDiv(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DivEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 305:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3160
		{
			yyVAL.node = assign.NewConcat(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ConcatEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 306:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3172
		{
			yyVAL.node = assign.NewMod(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ModEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 307:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3184
		{
			yyVAL.node = assign.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AndEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 308:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3196
		{
			yyVAL.node = assign.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OrEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 309:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3208
		{
			yyVAL.node = assign.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.XorEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 310:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3220
		{
			yyVAL.node = assign.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SlEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 311:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3232
		{
			yyVAL.node = assign.NewShiftRight(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SrEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 312:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3244
		{
			yyVAL.node = expr.NewPostInc(yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IncToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 313:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3256
		{
			yyVAL.node = expr.NewPreInc(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IncToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 314:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3268
		{
			yyVAL.node = expr.NewPostDec(yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DecToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 315:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3280
		{
			yyVAL.node = expr.NewPreDec(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DecToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 316:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3292
		{
			yyVAL.node = binary.NewBooleanOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.BooleanOrToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 317:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3304
		{
			yyVAL.node = binary.NewBooleanAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.BooleanAndToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 318:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3316
		{
			yyVAL.node = binary.NewLogicalOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.LogicalOrToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 319:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3328
		{
			yyVAL.node = binary.NewLogicalAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.LogicalAndToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 320:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3340
		{
			yyVAL.node = binary.NewLogicalXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.LogicalXorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 321:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3352
		{
			yyVAL.node = binary.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.VerticalBarToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 322:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3364
		{
			yyVAL.node = binary.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AmpersandToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 323:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3376
		{
			yyVAL.node = binary.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.CaretToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 324:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3388
		{
			yyVAL.node = binary.NewConcat(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DotToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 325:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3400
		{
			yyVAL.node = binary.NewPlus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PlusToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 326:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3412
		{
			yyVAL.node = binary.NewMinus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.MinusToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 327:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3424
		{
			yyVAL.node = binary.NewMul(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AsteriskToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 328:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3436
		{
			yyVAL.node = binary.NewPow(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PowToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 329:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3448
		{
			yyVAL.node = binary.NewDiv(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SlashToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 330:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3460
		{
			yyVAL.node = binary.NewMod(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PercentToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 331:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3472
		{
			yyVAL.node = binary.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SlToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 332:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3484
		{
			yyVAL.node = binary.NewShiftRight(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SrToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 333:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3496
		{
			yyVAL.node = expr.NewUnaryPlus(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.PlusToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 334:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3508
		{
			yyVAL.node = expr.NewUnaryMinus(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.MinusToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 335:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3520
		{
			yyVAL.node = expr.NewBooleanNot(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ExclamationMarkToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 336:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3532
		{
			yyVAL.node = expr.NewBitwiseNot(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.TildeToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 337:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3544
		{
			yyVAL.node = binary.NewIdentical(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsIdenticalToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 338:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3556
		{
			yyVAL.node = binary.NewNotIdentical(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsNotIdenticalToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 339:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3568
		{
			yyVAL.node = binary.NewEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 340:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3580
		{
			yyVAL.node = binary.NewNotEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsNotEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 341:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3592
		{
			yyVAL.node = binary.NewSmaller(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.LessToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 342:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3604
		{
			yyVAL.node = binary.NewSmallerOrEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsSmallerOrEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 343:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3616
		{
			yyVAL.node = binary.NewGreater(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.GreaterToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 344:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3628
		{
			yyVAL.node = binary.NewGreaterOrEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.IsGreaterOrEqualToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 345:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3640
		{
			yyVAL.node = binary.NewSpaceship(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.SpaceshipToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 346:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3652
		{
			yyVAL.node = expr.NewInstanceOf(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.InstanceofToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 347:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3664
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 348:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:3674
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 349:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:3680
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, yyDollar[3].node, yyDollar[5].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[5].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.QuestionMarkToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 350:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:3693
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, nil, yyDollar[4].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.QuestionMarkToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.ColonToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 351:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3706
		{
			yyVAL.node = binary.NewCoalesce(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.CoalesceToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 352:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:3718
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 353:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3724
		{
			yyVAL.node = cast.NewInt(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IntCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 354:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3736
		{
			yyVAL.node = cast.NewDouble(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DoubleCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 355:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3748
		{
			yyVAL.node = cast.NewString(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StringCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 356:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3760
		{
			yyVAL.node = cast.NewArray(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ArrayCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 357:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3772
		{
			yyVAL.node = cast.NewObject(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ObjectCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 358:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3784
		{
			yyVAL.node = cast.NewBool(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.BoolCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 359:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3796
		{
			yyVAL.node = cast.NewUnset(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.UnsetCastToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 360:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3808
		{
			if strings.EqualFold(yyDollar[1].token.Value, "die") {
				yyVAL.node = expr.NewDie(nil)
				if yyDollar[2].node != nil {
					yyVAL.node.(*expr.Die).Expr = yyDollar[2].node.(*expr.Exit).Expr
				}
			} else {
				yyVAL.node = expr.NewExit(nil)
				if yyDollar[2].node != nil {
					yyVAL.node.(*expr.Exit).Expr = yyDollar[2].node.(*expr.Exit).Expr
				}
			}

			// save position
			if yyDollar[2].node == nil {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			} else {
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			}

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ExitToken)

			if yyDollar[2].node != nil {
				yylex.(*Parser).comments.AddFromChildNode(yyVAL.node, yyDollar[2].node)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 361:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3838
		{
			yyVAL.node = expr.NewErrorSuppress(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.AtToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 362:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:3850
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 363:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:3856
		{
			yyVAL.node = expr.NewShellExec(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.BackquoteToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.BackquoteToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 364:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3869
		{
			yyVAL.node = expr.NewPrint(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.PrintToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 365:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:3881
		{
			yyVAL.node = expr.NewYield(nil, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.YieldToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 366:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3893
		{
			yyVAL.node = expr.NewYield(nil, yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.YieldToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 367:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:3905
		{
			yyVAL.node = expr.NewYield(yyDollar[2].node, yyDollar[4].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.YieldToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.DoubleArrowToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 368:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:3918
		{
			yyVAL.node = expr.NewYieldFrom(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.YieldFromToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 369:
		yyDollar = yyS[yypt-11 : yypt+1]
		//line php7/php7.y:3930
		{
			yyVAL.node = expr.NewClosure(yyDollar[5].list, yyDollar[7].ClosureUse, yyDollar[8].node, yyDollar[10].list, false, yyDollar[2].token != nil, yyDollar[3].str)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FunctionToken)
			if yyDollar[2].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.AmpersandToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[9].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[11].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 370:
		yyDollar = yyS[yypt-12 : yypt+1]
		//line php7/php7.y:3949
		{
			yyVAL.node = expr.NewClosure(yyDollar[6].list, yyDollar[8].ClosureUse, yyDollar[9].node, yyDollar[11].list, true, yyDollar[3].token != nil, yyDollar[4].str)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[12].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StaticToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.FunctionToken)
			if yyDollar[3].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.AmpersandToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[7].token, comment.CloseParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[10].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[12].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 371:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:3972
		{
			yyVAL.str = yylex.(*Parser).PhpDocComment
			yylex.(*Parser).PhpDocComment = ""

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 372:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:3982
		{
			yyVAL.token = nil
		}
	case 373:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:3986
		{
			yyVAL.token = yyDollar[1].token
		}
	case 374:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:3993
		{
			yyVAL.ClosureUse = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 375:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:3999
		{
			yyVAL.ClosureUse = expr.NewClosureUse(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.ClosureUse, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.ClosureUse, yyDollar[1].token, comment.UseToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.ClosureUse, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.ClosureUse, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 376:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4016
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 377:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4025
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 378:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4034
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(identifier)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 379:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4048
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[2].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewReference(variable)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.AmpersandToken)
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[2].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 380:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4068
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].node.(*node.ArgumentList))

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[2].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 381:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4077
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 382:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4089
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 383:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4101
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].node.(*node.ArgumentList))

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[2].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 384:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4113
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StaticToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 385:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4125
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 386:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4134
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 387:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4140
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 388:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:4149
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 389:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4155
		{
			yyVAL.node = expr.NewExit(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 390:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:4171
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 391:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4177
		{
			yyVAL.list = []node.Node{scalar.NewEncapsedStringPart(yyDollar[1].token.Value)}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 392:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4183
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 393:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:4192
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 394:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4198
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 395:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4207
		{
			yyVAL.node = expr.NewArray(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ArrayToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 396:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4221
		{
			yyVAL.node = expr.NewShortArray(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 397:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4234
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ConstantEncapsedStringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 398:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4249
		{
			yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.LnumberToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 399:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4261
		{
			yyVAL.node = scalar.NewDnumber(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DnumberToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 400:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4273
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.LineToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 401:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4285
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FileToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 402:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4297
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DirToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 403:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4309
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.TraitCToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 404:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4321
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.MethodCToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 405:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4333
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.FuncCToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 406:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4345
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NsCToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 407:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4357
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.ClassCToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 408:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4369
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, []node.Node{encapsed})

			// save position
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StartHeredocToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 409:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4383
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, nil)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StartHeredocToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 410:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4395
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DoubleQuoteToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 411:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4407
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, yyDollar[2].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StartHeredocToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 412:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4419
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 413:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4425
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 414:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4434
		{
			yyVAL.node = expr.NewConstFetch(yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 415:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4443
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)

			// save position
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)
			yylex.(*Parser).comments.AddFromToken(target, yyDollar[3].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 416:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4458
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)

			// save position
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)
			yylex.(*Parser).comments.AddFromToken(target, yyDollar[3].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 417:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4476
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 418:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4482
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 419:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:4491
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 420:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4497
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 421:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4506
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 422:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4515
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 423:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4521
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 424:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4531
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 425:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4540
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 426:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4546
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 427:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4556
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 428:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4565
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 429:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4571
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 430:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4584
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 431:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4597
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 432:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4610
		{
			yyVAL.node = expr.NewMethodCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ObjectOperatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 433:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4622
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 434:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4631
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 435:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4637
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 436:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4643
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ObjectOperatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 437:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4658
		{
			name := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(name)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 438:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4672
		{
			yyVAL.node = expr.NewVariable(yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DollarToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 439:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4686
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DollarToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 440:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4701
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 441:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4713
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 442:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4728
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 443:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4734
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 444:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4747
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 445:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4760
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ObjectOperatorToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 446:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4772
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 447:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4784
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.PaamayimNekudotayimToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 448:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4799
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IdentifierToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 449:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4811
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 450:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4821
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 451:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4830
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 452:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4842
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.OpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 453:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4852
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 454:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4861
		{
			if len(yyDollar[1].list) == 1 && yyDollar[1].list[0] == nil {
				yyVAL.list = yyDollar[1].list[:0]
			} else {
				yyVAL.list = yyDollar[1].list
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 455:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:4874
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 456:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4880
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 457:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4889
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			if lastNode(yyDollar[1].list) != nil {
				yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 458:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4900
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 459:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:4909
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DoubleArrowToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 460:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:4921
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[1].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 461:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4930
		{
			reference := expr.NewReference(yyDollar[4].node)
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, reference)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DoubleArrowToken)
			yylex.(*Parser).comments.AddFromToken(reference, yyDollar[3].token, comment.AmpersandToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 462:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4944
		{
			reference := expr.NewReference(yyDollar[2].node)
			yyVAL.node = expr.NewArrayItem(nil, reference)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(reference, yyDollar[1].token, comment.AmpersandToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 463:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:4957
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[5].list)
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, list)

			// save position
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[3].token, yyDollar[6].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.DoubleArrowToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[3].token, comment.ListToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[4].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[6].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 464:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:4975
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[3].list)
			yyVAL.node = expr.NewArrayItem(nil, list)

			// save position
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[1].token, comment.ListToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(list, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 465:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:4995
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 466:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5001
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yyVAL.list = append(yyDollar[1].list, encapsed)

			// save position
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(encapsed, yyDollar[2].token, comment.EncapsedAndWhitespaceToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 467:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5014
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 468:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5020
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{encapsed, yyDollar[2].node}

			// save position
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(encapsed, yyDollar[1].token, comment.EncapsedAndWhitespaceToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 469:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5036
		{
			name := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(name)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 470:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:5050
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseSquareBracket)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 471:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:5068
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			fetch := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewPropertyFetch(variable, fetch)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(fetch, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[1].token, comment.VariableToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.ObjectOperatorToken)
			yylex.(*Parser).comments.AddFromToken(fetch, yyDollar[3].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 472:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:5088
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DollarOpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 473:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:5101
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = expr.NewVariable(name)

			// save position
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DollarOpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(name, yyDollar[2].token, comment.StringVarnameToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 474:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:5117
		{
			identifier := node.NewIdentifier(yyDollar[2].token.Value)
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[4].node)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.DollarOpenCurlyBracesToken)
			yylex.(*Parser).comments.AddFromToken(variable, yyDollar[2].token, comment.StringVarnameToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[3].token, comment.OpenSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CloseSquareBracket)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[6].token, comment.CloseCurlyBracesToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 475:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:5137
		{
			yyVAL.node = yyDollar[2].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 476:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5146
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.StringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 477:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5158
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[1].token.Value); err == nil {
				yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)
			} else {
				yyVAL.node = scalar.NewString(yyDollar[1].token.Value)
			}

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.NumStringToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 478:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5175
		{
			var lnumber *scalar.Lnumber
			// TODO: add option to handle 64 bit integer
			_, err := strconv.Atoi(yyDollar[2].token.Value)
			isInt := err == nil

			if isInt {
				lnumber = scalar.NewLnumber(yyDollar[2].token.Value)
				yyVAL.node = expr.NewUnaryMinus(lnumber)
			} else {
				yyDollar[2].token.Value = "-" + yyDollar[2].token.Value
				yyVAL.node = scalar.NewString(yyDollar[2].token.Value)
			}

			// save position
			if isInt {
				yylex.(*Parser).positions.AddPosition(lnumber, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.MinusToken)
			if isInt {
				yylex.(*Parser).comments.AddFromToken(lnumber, yyDollar[2].token, comment.NumStringToken)
			} else {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.NumStringToken)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 479:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5206
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(identifier)

			// save position
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.VariableToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 480:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:5223
		{
			yyVAL.node = expr.NewIsset(yyDollar[3].list)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IssetToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			if yyDollar[4].token != nil {
				yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CommaToken)
			}
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[5].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 481:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:5240
		{
			yyVAL.node = expr.NewEmpty(yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.EmptyToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 482:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5254
		{
			yyVAL.node = expr.NewInclude(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IncludeToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 483:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5266
		{
			yyVAL.node = expr.NewIncludeOnce(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.IncludeOnceToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 484:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:5278
		{
			yyVAL.node = expr.NewEval(yyDollar[3].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.EvalToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[2].token, comment.OpenParenthesisToken)
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[4].token, comment.CloseParenthesisToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 485:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5292
		{
			yyVAL.node = expr.NewRequire(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.RequireToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 486:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:5304
		{
			yyVAL.node = expr.NewRequireOnce(yyDollar[2].node)

			// save position
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).comments.AddFromToken(yyVAL.node, yyDollar[1].token, comment.RequireOnceToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 487:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5319
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 488:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:5325
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).comments.AddFromToken(lastNode(yyDollar[1].list), yyDollar[2].token, comment.CommaToken)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 489:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:5337
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	}
	goto yystack /* stack new state and value */
}
